# this file to store possible shortform, normalized, prefix, postfix, tatabahasa, titles and stopwords
# 1. kata tanya, tanya_list
# 2. kata perintah, perintah_list
# 3. kata pangkal, pangkal_list
# 4. kata bantu, bantu_list
# 5. kata penguat, penguat_list
# 6. kata penegas, penegas_list
# 7. kata nafi, nafi_list
# 8. kata pemeri, pemeri_list
# 9. kata sendi, sendi_list
# 10. kata pembenar, pembenar_list
# 11. number, nombor_list
# 12. suku bilangan, suku_bilangan_list
# 13. kata penerangan, keterangan_list
# 14. kata arah, arah_list
# 15. kata hubung, hubung_list
# 16. ganti nama, gantinama_list
# 17. prefix, permulaan
# 18. postfix, hujung
# 19. postfix malaysian slang, hujung_malaysian
# 20. sound to word, sounds
# 21. word replacement for date string, date_replace
# 22. word replacement for local slang, rules_normalizer
# 23. calon dictionary, calon_dictionary
# 24. stopwords, stopwords
# 25. bi_vowels, bivowels, example, `aa`
# 26. group_compound, group for biconsonant, example, `ng`

GO = 0
PAD = 1
EOS = 2
UNK = 3

alphabet = 'qwertyuiopasdfghjklzxcvbnm'
consonants = 'bcdfghjklmnpqrstvwxyz'
vowels = 'aeiou'

tanya_list = [
    'kenapa',
    'bila',
    'siapa',
    'mengapa',
    'apa',
    'bagaimana',
    'berapa',
    'mana',
]
perintah_list = ['jangan', 'sila', 'tolong', 'harap', 'usah', 'jemput', 'minta']
pangkal_list = [
    'maka',
    'alkisah',
    'arakian',
    'syahdah',
    'adapun',
    'bermula',
    'kalakian',
]
bantu_list = [
    'akan',
    'telah',
    'boleh',
    'mesti',
    'belum',
    'sudah',
    'dapat',
    'masih',
    'harus',
    'hendak',
]
penguat_list = [
    'paling',
    'agak',
    'sungguh',
    'amat',
    'terlalu',
    'nian',
    'benar',
    'paling',
]
penegas_list = ['jua', 'juga', 'sahaja', 'hanya', 'memang', 'lagi', 'pun']
nafi_list = ['bukan', 'tidak', 'tak', 'tiada', 'tidaklah', 'tidakkah']
pemeri_list = ['ialah', 'adalah']
sendi_list = [
    'akan',
    'kepada',
    'terhadap',
    'bagi',
    'untuk',
    'dari',
    'daripada',
    'di',
    'dengan',
    'hingga',
    'sampai',
    'ke',
    'kepada',
    'oleh',
    'pada',
    'sejak',
    'seperti',
    'umpama',
    'bak',
    'tentang',
    'laksanabagai',
    'semenjak',
    'dalam',
    'antara',
]
pembenar_list = ['ya', 'benar', 'betul']
nombor_list = [
    'satu',
    'dua',
    'tiga',
    'empat',
    'lima',
    'enam',
    'tujuh',
    'lapan',
    'sembilan',
    'kosong',
]
suku_bilangan_list = ['per', 'suku', 'setengah', 'separuh', 'tiga suku']
pisahan_list = ['setiap', 'tiap']
keterangan_list = [
    'begitu',
    'begini',
    'demikian',
    'perlahan',
    'cepat',
    'lena',
    'akan',
    'sedang',
    'belum',
    'telah',
    'sekarang',
    'sebentar',
    'semalam',
    'mungkin',
    'agak',
    'barangkali',
    'pasti',
    'tentu',
    'sudah',
    'selalu',
    'kadang',
    'acapkali',
    'sesekali',
    'yang',
]
arah_list = [
    'atas',
    'bawah',
    'tepi',
    'antara',
    'hadapan',
    'utara',
    'sisi',
    'luar',
]
hubung_list = [
    'agar',
    'apabila',
    'atau',
    'bahawa',
    'dan',
    'hingga',
    'jika',
    'jikalau',
    'kecuali',
    'kerana',
    'lalu',
    'manakala',
    'sambil',
    'serta',
    'semenjak',
    'sementara',
    'sungguhpun',
    'supaya',
    'walaupun',
    'tetapi',
    'berkenan',
    'berkenaan',
]
gantinama_list = [
    'aku',
    'saya',
    'hamba',
    'patik',
    'beta',
    'kami',
    'kita',
    'anda',
    'awak',
    'engkau',
    'tuanku',
    'kalian',
    'kamu',
    'baginda',
    'beliau',
    'mereka',
    'ini',
    'itu',
    'sini',
    'situ',
    'sana',
    'kini',
    'dia',
    'rm',
]

permulaan = {
    'bel': 'bel',
    'se': 'se',
    'see': 'se',
    'ter': 'ter',
    'terr': 'ter',
    'men': 'men',
    'menn': 'men',
    'meng': 'meng',
    'mengg': 'meng',
    'mem': 'mem',
    'mm': 'mem',
    'memper': 'memper',
    'di': 'di',
    'ddi': 'di',
    'pe': 'pe',
    'ppe': 'pe',
    'ppee': 'pe',
    'me': 'me',
    'mme': 'mme',
    'ke': 'ke',
    'kee': 'ke',
    'ber': 'ber',
    'berr': 'ber',
    'pen': 'pen',
    'penn': 'pen',
    'per': 'per',
    'perr': 'perr',
}

hujung = {
    'kan': 'kan',
    'kann': 'kan',
    'kah': 'kah',
    'kahh': 'kah',
    'lah': 'lah',
    'lahh': 'lah',
    'tah': 'tah',
    'tahh': 'tah',
    'nya': 'nya',
    'nyaa': 'nya',
    'an': 'an',
    'ann': 'an',
    'wan': 'wan',
    'wann': 'wan',
    'wati': 'wati',
    'watii': 'wati',
    'ita': 'ita',
    'itaa': 'ita',
    'nye': 'nya',
    'nyee': 'nya',
    'nyo': 'nya',
    'nyoo': 'nya',
    'loh': 'lah',
    'lohh': 'lah',
}

hujung_malaysian = ['lah', 'la', 'ler']

sounds = {
    'x': 'tidak',
    'y': 'kenapa',
    'n': 'dan',
    'g': 'pergi',
    's': 'seperti',
    'd': 'di',
    'k': 'ok',
    'u': 'awak',
    't': 'nanti',
    'p': 'pergi',
    'wai': 'kenapa',
    'i': 'saya',
}

tatabahasa_dict = {
    'KT': tanya_list,
    'KP': perintah_list,
    'KPA': pangkal_list,
    'KB': bantu_list,
    'KPENGUAT': penguat_list,
    'KPENEGAS': penegas_list,
    'NAFI': nafi_list,
    'KPEMERI': pemeri_list,
    'KS': sendi_list,
    'KPEMBENAR': pembenar_list,
    'NO': nombor_list,
    'SUKU': suku_bilangan_list,
    'PISAHAN': pisahan_list,
    'KETERANGAN': keterangan_list,
    'ARAH': arah_list,
    'KH': hubung_list,
    'GN': gantinama_list,
}

stopword_tatabahasa = list(
    set(
        tanya_list
        + perintah_list
        + pangkal_list
        + bantu_list
        + penguat_list
        + penegas_list
        + nafi_list
        + pemeri_list
        + sendi_list
        + pembenar_list
        + nombor_list
        + suku_bilangan_list
        + pisahan_list
        + keterangan_list
        + arah_list
        + hubung_list
        + gantinama_list
    )
)

date_replace = {
    'awk': 'awak',
    'ngkau': 'engkau',
    'lps': 'lalu',
    'lepas': 'lalu',
    'mnggu': 'minggu',
    'bln': 'bulan',
    'skrg': 'sekarang',
    'thun': 'tahun',
    'hri': 'hari',
    'minute': 'minit',
    'mnit': 'minit',
    'sec': 'saat',
    'second': 'saat',
    'yesterday': 'semalam',
    'kelmarin': 'kelmarin',
    'smalam': 'semalam',
    'dpan': 'depan',
    'dpn': 'depan',
    'esk': 'esok',
    'pagi': 'AM',
    'pgi': 'AM',
    'morning': 'AM',
    'tengahari': 'PM',
    'tngahari': 'PM',
    'petang': 'PM',
    'ptg': 'PM',
    'malam': 'PM',
    'semalam': 'semalam',
    'pkul': 'pukul',
}
rules_normalizer = {
    'ty': 'terima kasih',
    'january': 'januari',
    'february': 'februari',
    'march': 'mac',
    'may': 'mei',
    'june': 'jun',
    'july': 'julai',
    'august': 'ogos',
    'october': 'oktober',
    'december': 'disember',
    'dec': 'dis',
    'oct': 'okt',
    'monday': 'isnin',
    'mon': 'isn',
    'tuesday': 'selasa',
    'tues': 'sel',
    'wednesday': 'rabu',
    'wed': 'rab',
    'thursday': 'khamis',
    'thurs': 'kha',
    'friday': 'jumaat',
    'fri': 'jum',
    'saturday': 'sabtu',
    'sat': 'sab',
    'sunday': 'ahad',
    'sun': 'ahd',
    'gemen': 'kerajaan',
    'camtu': 'macam itu',
    'experience': 'pengalaman',
    'kpd': 'kepada',
    'bengng': 'bengang',
    'mntak': 'minta',
    'bagasi': 'bagasi',
    'kg': 'kampung',
    'kilo': 'kilogram',
    'g': 'pergi',
    'grm': 'gram',
    'k': 'okay',
    'abgkat': 'abang dekat',
    'abis': 'habis',
    'ade': 'ada',
    'adoi': 'aduh',
    'adoii': 'aduh',
    'aerodarat': 'kapal darat',
    'agkt': 'angkat',
    'ahh': 'ah',
    'ailior': 'air liur',
    'airasia': 'air asia x',
    'airasiax': 'penerbangan',
    'airline': 'penerbangan',
    'airlines': 'penerbangan',
    'airport': 'lapangan terbang',
    'airpot': 'lapangan terbang',
    'aje': 'sahaja',
    'ajelah': 'sahajalah',
    'ajer': 'sahaja',
    'ak': 'aku',
    'aq': 'aku',
    'all': 'semua',
    'ambik': 'ambil',
    'amek': 'ambil',
    'amer': 'amir',
    'amik': 'ambil',
    'ana': 'saya',
    'angkt': 'angkat',
    'anual': 'tahunan',
    'apapun': 'apa pun',
    'ape': 'apa',
    'arab': 'arab',
    'area': 'kawasan',
    'aritu': 'hari itu',
    'ask': 'tanya',
    'astro': 'astro',
    'at': 'pada',
    'attitude': 'sikap',
    'babi': 'khinzir',
    'back': 'belakang',
    'bag': 'beg',
    'bang': 'abang',
    'bangla': 'bangladesh',
    'banyk': 'banyak',
    'bard': 'pujangga',
    'bargasi': 'bagasi',
    'bawak': 'bawa',
    'bawanges': 'bawang',
    'be': 'jadi',
    'behave': 'berkelakuan baik',
    'belagak': 'berlagak',
    'berdisiplin': 'berdisplin',
    'berenti': 'berhenti',
    'beskal': 'basikal',
    'bff': 'rakan karib',
    'bg': 'bagi',
    'bgi': 'bagi',
    'biase': 'biasa',
    'big': 'besar',
    'bike': 'basikal',
    'bile': 'bila',
    'binawe': 'binatang',
    'bini': 'isteri',
    'bkn': 'bukan',
    'bla': 'bila',
    'blom': 'belum',
    'bnyak': 'banyak',
    'body': 'tubuh',
    'bole': 'boleh',
    'boss': 'bos',
    'bowling': 'boling',
    'bpe': 'berapa',
    'brand': 'jenama',
    'brg': 'barang',
    'briefing': 'taklimat',
    'brng': 'barang',
    'bro': 'abang',
    'bru': 'baru',
    'bruntung': 'beruntung',
    'bsikal': 'basikal',
    'btnggjwb': 'bertanggungjawab',
    'btul': 'betul',
    'buatlh': 'buatlah',
    'buh': 'letak',
    'buka': 'buka',
    'but': 'tetapi',
    'bwk': 'bawa',
    'by': 'dengan',
    'byr': 'bayar',
    'bz': 'sibuk',
    'camera': 'kamera',
    'camni': 'macam ini',
    'cane': 'macam mana',
    'cant': 'tak boleh',
    'carakerja': 'cara kerja',
    'care': 'jaga',
    'cargo': 'kargo',
    'cctv': 'kamera litar tertutup',
    'celako': 'celaka',
    'cer': 'cerita',
    'cheap': 'murah',
    'check': 'semak',
    'ciput': 'sedikit',
    'cite': 'cerita',
    'citer': 'cerita',
    'ckit': 'sikit',
    'cikit': 'sikit',
    'ckp': 'cakap',
    'class': 'kelas',
    'cm': 'macam',
    'cmni': 'macam ini',
    'cmpak': 'campak',
    'committed': 'komited',
    'company': 'syarikat',
    'complain': 'aduan',
    'corn': 'jagung',
    'couldnt': 'tak boleh',
    'cr': 'cari',
    'crew': 'krew',
    'cube': 'cuba',
    'cuma': 'cuma',
    'curinyaa': 'curinya',
    'cust': 'pelanggan',
    'customer': 'pelanggan',
    'd': 'di',
    'da': 'dah',
    'dn': 'dan',
    'dahh': 'dah',
    'damaged': 'rosak',
    'dapek': 'dapat',
    'day': 'hari',
    'dazrin': 'dazrin',
    'dbalingnya': 'dibalingnya',
    'de': 'ada',
    'deep': 'dalam',
    'deliberately': 'sengaja',
    'depa': 'mereka',
    'dessa': 'desa',
    'dgn': 'dengan',
    'dh': 'dah',
    'didunia': 'di dunia',
    'diorang': 'mereka',
    'diorng': 'mereka',
    'direct': 'secara terus',
    'diving': 'junam',
    'dkt': 'dekat',
    'dlempar': 'dilempar',
    'dlm': 'dalam',
    'dlt': 'padam',
    'dlu': 'dulu',
    'done': 'siap',
    'dont': 'jangan',
    'dorg': 'mereka',
    'dpermudhkn': 'dipermudahkan',
    'dpt': 'dapat',
    'dri': 'dari',
    'dsb': 'dan sebagainya',
    'dy': 'dia',
    'educate': 'mendidik',
    'ensure': 'memastikan',
    'everything': 'semua',
    'ewahh': 'wah',
    'expect': 'sangka',
    'fb': 'facebook',
    'fired': 'pecat',
    'first': 'pertama',
    'fkr': 'fikir',
    'flight': 'kapal terbang',
    'for': 'untuk',
    'free': 'percuma',
    'friend': 'kawan',
    'fyi': 'untuk pengetahuan anda',
    'gantila': 'gantilah',
    'gantirugi': 'ganti rugi',
    'gentlemen': 'lelaki budiman',
    'gerenti': 'jaminan',
    'gile': 'gila',
    'gk': 'juga',
    'gnti': 'ganti',
    'go': 'pergi',
    'gomen': 'kerajaan',
    'goment': 'kerajaan',
    'good': 'baik',
    'ground': 'tanah',
    'guarno': 'macam mana',
    'hampa': 'mereka',
    'hampeh': 'teruk',
    'hanat': 'jahanam',
    'handle': 'kawal',
    'handling': 'kawalan',
    'hanta': 'hantar',
    'haritu': 'hari itu',
    'harini': 'hari ini',
    'hate': 'benci',
    'have': 'ada',
    'hawau': 'celaka',
    'henpon': 'telefon',
    'heran': 'hairan',
    'him': 'dia',
    'his': 'dia',
    'hmpa': 'mereka',
    'hntr': 'hantar',
    'hotak': 'otak',
    'hr': 'hari',
    'i': 'saya',
    'hrga': 'harga',
    'hrp': 'harap',
    'hu': 'sedih',
    'humble': 'merendah diri',
    'ibon': 'ikon',
    'ichi': 'inci',
    'idung': 'hidung',
    'if': 'jika',
    'ig': 'instagram',
    'iklas': 'ikhlas',
    'improve': 'menambah baik',
    'in': 'masuk',
    'isn t': 'tidak',
    'isyaallah': 'insyallah',
    'ja': 'sahaja',
    'japan': 'jepun',
    'jd': 'jadi',
    'saja': 'sahaja',
    'saje': 'sahaja',
    'je': 'sahaja',
    'jee': 'sahaja',
    'jek': 'sahaja',
    'jepun': 'jepun',
    'jer': 'sahaja',
    'jerr': 'sahaja',
    'jez': 'sahaja',
    'jg': 'juga',
    'jgk': 'juga',
    'jgn': 'jangan',
    'jgnla': 'janganlah',
    'jibake': 'celaka',
    'jjur': 'jujur',
    'job': 'kerja',
    'jobscope': 'skop kerja',
    'jogja': 'jogjakarta',
    'jpam': 'jpam',
    'jth': 'jatuh',
    'jugak': 'juga',
    'ka': 'ke',
    'kalo': 'kalau',
    'kalu': 'kalau',
    'kang': 'nanti',
    'kantoi': 'temberang',
    'kasi': 'beri',
    'kat': 'dekat',
    'kbye': 'ok bye',
    'kearah': 'ke arah',
    'kecik': 'kecil',
    'keja': 'kerja',
    'keje': 'kerja',
    'kejo': 'kerja',
    'keksongan': 'kekosongan',
    'kemana': 'ke mana',
    'kene': 'kena',
    'kenekan': 'kenakan',
    'kesah': 'kisah',
    'ketempat': 'ke tempat',
    'kije': 'kerja',
    'kijo': 'kerja',
    'kiss': 'cium',
    'kite': 'kita',
    'kito': 'kita',
    'kje': 'kerja',
    'kjr': 'kerja',
    'kk': 'okay',
    'kmi': 'kami',
    'kt': 'kat',
    'tlg': 'tolong',
    'kl': 'kuala lumpur',
    'klai': 'kalau',
    'klau': 'kalau',
    'klia': 'klia',
    'klo': 'kalau',
    'klu': 'kalau',
    'kn': 'kan',
    'knapa': 'kenapa',
    'kne': 'kena',
    'ko': 'kau',
    'kompom': 'sah',
    'korang': 'kamu semua',
    'korea': 'korea',
    'korg': 'kamu semua',
    'kot': 'mungkin',
    'krja': 'kerja',
    'ksalahan': 'kesalahan',
    'kta': 'kita',
    'kuar': 'keluar',
    'kut': 'mungkin',
    'la': 'lah',
    'laa': 'lah',
    'lahabau': 'celaka',
    'lahanat': 'celaka',
    'lainda': 'lain dah',
    'lak': 'pula',
    'last': 'akhir',
    'le': 'lah',
    'leader': 'ketua',
    'leave': 'pergi',
    'ler': 'lah',
    'less': 'kurang',
    'letter': 'surat',
    'lg': 'lagi',
    'lgi': 'lagi',
    'lngsong': 'langsung',
    'lol': 'hehe',
    'lorr': 'lah',
    'low': 'rendah',
    'lps': 'lepas',
    'luggage': 'bagasi',
    'lumbe': 'lumba',
    'lyak': 'layak',
    'maap': 'maaf',
    'maapkan': 'maafkan',
    'mahai': 'mahal',
    'mampos': 'mampus',
    'mart': 'kedai',
    'mau': 'mahu',
    'mcm': 'macam',
    'mcmtu': 'macam itu',
    'memerlukn': 'memerlukan',
    'mengembirakan': 'menggembirakan',
    'mengmbilnyer': 'mengambilnya',
    'mengtasi': 'mengatasi',
    'mg': 'memang',
    'mihak': 'memihak',
    'min': 'admin',
    'mingu': 'minggu',
    'mintak': 'minta',
    'mjtuhkn': 'menjatuhkan',
    'mkyong': 'mak yong',
    'mlibatkn': 'melibatkan',
    'mmg': 'memang',
    'mmnjang': 'memanjang',
    'mmpos': 'mampus',
    'mn': 'mana',
    'mna': 'mana',
    'mntak': 'minta',
    'mntk': 'minta',
    'mnyusun': 'menyusun',
    'mood': 'suasana',
    'most': 'paling',
    'mr': 'tuan',
    'msa': 'masa',
    'msia': 'malaysia',
    'mst': 'mesti',
    'mu': 'awak',
    'much': 'banyak',
    'muko': 'muka',
    'mum': 'emak',
    'n': 'dan',
    'nah': 'nah',
    'nanny': 'nenek',
    'napo': 'kenapa',
    'nati': 'nanti',
    'ngan': 'dengan',
    'ngn': 'dengan',
    'ni': 'ini',
    'nie': 'ini',
    'nii': 'ini',
    'nk': 'nak',
    'nmpk': 'nampak',
    'nye': 'nya',
    'ofis': 'pejabat',
    'ohh': 'oh',
    'oii': 'hoi',
    'one': 'satu',
    'online': 'dalam talian',
    'or': 'atau',
    'org': 'orang',
    'orng': 'orang',
    'otek': 'otak',
    'p': 'pergi',
    'paid': 'dah bayar',
    'palabana': 'kepala otak',
    'pasni': 'lepas ini',
    'passengers': 'penumpang',
    'passengger': 'penumpang',
    'pastu': 'lepas itu',
    'pd': 'pada',
    'pegi': 'pergi',
    'pekerje': 'pekerja',
    'pekrja': 'pekerja',
    'perabih': 'perabis',
    'perkerja': 'pekerja',
    'pg': 'pergi',
    'phuii': 'puih',
    'pikir': 'fikir',
    'pilot': 'juruterbang',
    'pk': 'fikir',
    'pkerja': 'pekerja',
    'pkerjaan': 'pekerjaan',
    'pki': 'pakai',
    'please': 'tolong',
    'pls': 'tolong',
    'pn': 'pun',
    'pnh': 'pernah',
    'pnt': 'penat',
    'pnya': 'punya',
    'pon': 'pun',
    'priority': 'keutamaan',
    'properties': 'harta benda',
    'ptugas': 'petugas',
    'pub': 'kelab malam',
    'pulak': 'pula',
    'puye': 'punya',
    'pwrcuma': 'percuma',
    'pyahnya': 'payahnya',
    'quality': 'kualiti',
    'quit': 'keluar',
    'ramly': 'ramly',
    'rege': 'harga',
    'reger': 'harga',
    'report': 'laporan',
    'resigned': 'meletakkan jawatan',
    'respect': 'hormat',
    'rizal': 'rizal',
    'rosak': 'rosak',
    'rosok': 'rosak',
    'rse': 'rasa',
    'sacked': 'buang',
    'sado': 'tegap',
    'salute': 'sanjung',
    'sam': 'sama',
    'same': 'sama',
    'samp': 'sampah',
    'sbb': 'sebab',
    'sbgai': 'sebagai',
    'sblm': 'sebelum',
    'sblum': 'sebelum',
    'sbnarnya': 'sebenarnya',
    'sbum': 'sebelum',
    'sdg': 'sedang',
    'sebb': 'sebab',
    'sebijik': 'sebiji',
    'see': 'lihat',
    'seen': 'dilihat',
    'selangor': 'selangor',
    'selfie': 'swafoto',
    'sempoi': 'cantik',
    'senaraihitam': 'senarai hitam',
    'seorg': 'seorang',
    'service': 'perkhidmatan',
    'sgt': 'sangat',
    'shared': 'kongsi',
    'shirt': 'kemeja',
    'shut': 'tutup',
    'sib': 'nasib',
    'skali': 'sekali',
    'sket': 'sikit',
    'sma': 'sama',
    'smoga': 'semoga',
    'smpoi': 'cantik',
    'sndiri': 'sendiri',
    'sndr': 'sendiri',
    'sndri': 'sendiri',
    'sne': 'sana',
    'so': 'jadi',
    'sop': 'tatacara pengendalian piawai',
    'sorang': 'seorang',
    'spoting': 'pembintikan',
    'sronok': 'seronok',
    'ssh': 'susah',
    'staff': 'staf',
    'standing': 'berdiri',
    'start': 'mula',
    'steady': 'mantap',
    'stiap': 'setiap',
    'stress': 'stres',
    'student': 'pelajar',
    'study': 'belajar',
    'studycase': 'kajian kes',
    'sure': 'pasti',
    'sykt': 'syarikat',
    'tah': 'entah',
    'taik': 'tahi',
    'takan': 'tak akan',
    'takat': 'setakat',
    'takde': 'tak ada',
    'takkan': 'tak akan',
    'taknak': 'tak nak',
    'tang': 'tentang',
    'tanggungjawab': 'bertanggungjawab',
    'taraa': 'sementara',
    'tau': 'tahu',
    'tbabit': 'terbabit',
    'team': 'pasukan',
    'terbaekk': 'terbaik',
    'teruknye': 'teruknya',
    'tgk': 'tengok',
    'that': 'itu',
    'thinking': 'fikir',
    'those': 'itu',
    'time': 'masa',
    'tk': 'tak',
    'tnggongjwb': 'tanggungjawab',
    'tngok': 'tengok',
    'tngu': 'tunggu',
    'to': 'kepada',
    'tosak': 'rosak',
    'tp': 'tapi',
    'tpi': 'tapi',
    'tpon': 'telefon',
    'transfer': 'pindah',
    'trgelak': 'tergelak',
    'ts': 'tan sri',
    'tstony': 'tan sri tony',
    'tu': 'itu',
    'tuh': 'itu',
    'tula': 'itulah',
    'umeno': 'umno',
    'unfortunately': 'malangnya',
    'unhappy': 'tidak gembira',
    'up': 'naik',
    'upkan': 'naikkan',
    'ur': 'awak',
    'utk': 'untuk',
    'very': 'sangat',
    'viral': 'tular',
    'vote': 'undi',
    'warning': 'amaran',
    'warranty': 'waranti',
    'wassap': 'whatsapp',
    'wat': 'apa',
    'weii': 'wei',
    'well': 'maklumlah',
    'win': 'menang',
    'with': 'dengan',
    'wt': 'buat',
    'x': 'tak',
    'tw': 'tahu',
    'ye': 'ya',
    'yee': 'ya',
    'yg': 'yang',
    'yng': 'yang',
    'you': 'awak',
    'your': 'awak',
    'sakai': 'selekeh',
    'rmb': 'billion ringgit',
    'rmj': 'juta ringgit',
    'rmk': 'ribu ringgit',
    'rm': 'ringgit',
    ':*': '<kiss>',
    ':-*': '<kiss>',
    ':x': '<kiss>',
    ':-)': '<happy>',
    ':-))': '<happy>',
    ':-)))': '<happy>',
    ':-))))': '<happy>',
    ':-)))))': '<happy>',
    ':-))))))': '<happy>',
    ':)': '<happy>',
    ':))': '<happy>',
    ':)))': '<happy>',
    ':))))': '<happy>',
    ':)))))': '<happy>',
    ':))))))': '<happy>',
    ':)))))))': '<happy>',
    ':o)': '<happy>',
    ':]': '<happy>',
    ':3': '<happy>',
    ':c)': '<happy>',
    ':>': '<happy>',
    '=]': '<happy>',
    '8)': '<happy>',
    '=)': '<happy>',
    ':}': '<happy>',
    ':^)': '<happy>',
    '|;-)': '<happy>',
    ":'-)": '<happy>',
    ":')": '<happy>',
    '\o/': '<happy>',
    '*\\0/*': '<happy>',
    ':-D': '<laugh>',
    ':D': '<laugh>',
    '8-D': '<laugh>',
    '8D': '<laugh>',
    'x-D': '<laugh>',
    'xD': '<laugh>',
    'X-D': '<laugh>',
    'XD': '<laugh>',
    '=-D': '<laugh>',
    '=D': '<laugh>',
    '=-3': '<laugh>',
    '=3': '<laugh>',
    'B^D': '<laugh>',
    '>:[': '<sad>',
    ':-(': '<sad>',
    ':-((': '<sad>',
    ':-(((': '<sad>',
    ':-((((': '<sad>',
    ':-(((((': '<sad>',
    ':-((((((': '<sad>',
    ':-(((((((': '<sad>',
    ':(': '<sad>',
    ':((': '<sad>',
    ':(((': '<sad>',
    ':((((': '<sad>',
    ':(((((': '<sad>',
    ':((((((': '<sad>',
    ':(((((((': '<sad>',
    ':((((((((': '<sad>',
    ':-c': '<sad>',
    ':c': '<sad>',
    ':-<': '<sad>',
    ':<': '<sad>',
    ':-[': '<sad>',
    ':[': '<sad>',
    ':{': '<sad>',
    ':-||': '<sad>',
    ':@': '<sad>',
    ":'-(": '<sad>',
    ":'(": '<sad>',
    'D:<': '<sad>',
    'D:': '<sad>',
    'D8': '<sad>',
    'D;': '<sad>',
    'D=': '<sad>',
    'DX': '<sad>',
    'v.v': '<sad>',
    "D-':": '<sad>',
    '(>_<)': '<sad>',
    ':|': '<sad>',
    '>:O': '<surprise>',
    ':-O': '<surprise>',
    ':-o': '<surprise>',
    ':O': '<surprise>',
    '°o°': '<surprise>',
    'o_O': '<surprise>',
    'o_0': '<surprise>',
    'o.O': '<surprise>',
    'o-o': '<surprise>',
    '8-0': '<surprise>',
    '|-O': '<surprise>',
    ';-)': '<wink>',
    ';)': '<wink>',
    '*-)': '<wink>',
    '*)': '<wink>',
    ';-]': '<wink>',
    ';]': '<wink>',
    ';D': '<wink>',
    ';^)': '<wink>',
    ':-,': '<wink>',
    '>:P': '<tong>',
    ':-P': '<tong>',
    ':P': '<tong>',
    'X-P': '<tong>',
    'x-p': '<tong>',
    'xp': '<tong>',
    'XP': '<tong>',
    ':-p': '<tong>',
    ':p': '<tong>',
    '=p': '<tong>',
    ':-Þ': '<tong>',
    ':Þ': '<tong>',
    ':-b': '<tong>',
    ':b': '<tong>',
    ':-&': '<tong>',
    '>:\\': '<annoyed>',
    '>:/': '<annoyed>',
    ':-/': '<annoyed>',
    ':-.': '<annoyed>',
    ':/': '<annoyed>',
    ':\\': '<annoyed>',
    '=/': '<annoyed>',
    '=\\': '<annoyed>',
    ':L': '<annoyed>',
    '=L': '<annoyed>',
    ':S': '<annoyed>',
    '>.<': '<annoyed>',
    ':-|': '<annoyed>',
    '<:-|': '<annoyed>',
    ':-X': '<seallips>',
    ':X': '<seallips>',
    ':-#': '<seallips>',
    ':#': '<seallips>',
    'O:-)': '<angel>',
    '0:-3': '<angel>',
    '0:3': '<angel>',
    '0:-)': '<angel>',
    '0:)': '<angel>',
    '0;^)': '<angel>',
    '>:)': '<devil>',
    '>:D': '<devil>',
    '>:-D': '<devil>',
    '>;)': '<devil>',
    '>:-)': '<devil>',
    '}:-)': '<devil>',
    '}:)': '<devil>',
    '3:-)': '<devil>',
    '3:)': '<devil>',
    'o/\o': '<highfive>',
    '^5': '<highfive>',
    '>_>^': '<highfive>',
    '^<_<': '<highfive>',
    '<3': '<heart>',
}
stopwords_calon = [
    'datuk',
    'bin',
    'hj',
    'haji',
    'bn',
    'bnt',
    'prof',
    'binti',
    'dr',
    'ustaz',
    'mejar',
    'ir',
    'md',
    'tuan',
    'puan',
    'yb',
    'ustazah',
    'cikgu',
    'dato',
    'dsp',
]

calon_dictionary = {
    'dr': 'Doktor',
    'yb': 'Yang Berhormat',
    'hj': 'Haji',
    'ybm': 'Yang Berhormat Mulia',
    'tyt': 'Tuan Yang Terutama',
    'yab': 'Yang Berhormat',
    'ybm': 'Yang Berhormat Mulia',
    'yabhg': 'Yang Amat Berbahagia',
    'ybhg': 'Yang Berbahagia',
    'miss': 'Cik',
}
stopwords = [
    'not',
    'ada',
    'inikah',
    'sampai',
    'adakah',
    'inilah',
    'sana',
    'adakan',
    'itu',
    'sangat',
    'adalah',
    'itukah',
    'sangatlah',
    'adanya',
    'itulah',
    'saya',
    'adapun',
    'jadi',
    'se',
    'agak',
    'jangan',
    'seandainya',
    'agar',
    'namun',
    'janganlah',
    'sebab',
    'akan',
    'cer',
    'jika',
    'sebagai',
    'aku',
    'usahlah',
    'begitu',
    'cuba',
    'jikalau',
    'sebagaimana',
    'akulah',
    'jua',
    'sebanyak',
    'akupun',
    'juapun',
    'dpt',
    'sebelum',
    'al',
    'juga',
    'sebelummu',
    'alangkah',
    'kalau',
    'sebelumnya',
    'allah',
    'kami',
    'sebenarnya',
    'amat',
    'tengku',
    'kamikah',
    'ckp',
    'mmg',
    'secara',
    'antara',
    'justeru',
    'kamipun',
    'sedang',
    'antaramu',
    'bin',
    'binti',
    'datuk',
    'dato',
    'jumlah',
    'kamu',
    'sedangkan',
    'antaranya',
    'kamukah',
    'sedikit',
    'apa',
    'kamupun',
    'sedikitpun',
    'apa-apa',
    'katakan',
    'segala',
    'dgn',
    'apabila',
    'ke',
    'takde',
    'kat',
    'takda',
    'sehingga',
    'apakah',
    'kecuali',
    'sejak',
    'apapun',
    'kelak',
    'sekalian',
    'atas',
    'tadi',
    'cer',
    'kembali',
    'sekalipun',
    'atasmu',
    'kemudian',
    'sekarang',
    'atasnya',
    'kepada',
    'sekitar',
    'atau',
    'kepadaku',
    'selain',
    'ataukah',
    'kepadakulah',
    'selalu',
    'ataupun',
    'kepadamu',
    'selama',
    'bagaimana',
    'kepadanya',
    'selama-lamanya',
    'bagaimanakah',
    'kepadanyalah',
    'seluruh',
    'bagi',
    'kerana',
    'seluruhnya',
    'bagimu',
    'kerananya',
    'sementara',
    'baginya',
    'kesan',
    'byk',
    'semua',
    'bahawa',
    'ketika',
    'semuanya',
    'bahawasanya',
    'wdir',
    'met',
    'dew',
    'kini',
    'haram',
    'haramm',
    'harem',
    'haremm',
    'semula',
    'bahkan',
    'manakala',
    'selepas',
    'kita',
    'senantiasa',
    'bahwa',
    'ku',
    'sendiri',
    'banyak',
    'kurang',
    'sentiasa',
    'banyaknya',
    'lagi',
    'seolah',
    'barangsiapa',
    'lain',
    'seolah-olah',
    'bawah',
    'lalu',
    'seorangpun',
    'beberapa',
    'ever',
    'got',
    'sane',
    'let',
    'eng',
    'dont',
    "'ve",
    'even',
    'lamanya',
    'separuh',
    'begitu',
    'langsung',
    'sepatutnya',
    'begitupun',
    'lebih',
    'seperti',
    'belaka',
    'maha',
    'seraya',
    'belum',
    'mahu',
    'sering',
    'belumkah',
    'mahukah',
    'pagi',
    'malam',
    'petang',
    'siang',
    'lepas',
    'awak',
    'serta',
    'berada',
    'mahupun',
    'seseorang',
    'berapa',
    'maka',
    'sesiapa',
    'berikan',
    'malah',
    'sesuatu',
    'beriman',
    'wasalam',
    'wsalam',
    'assalamualaikum',
    'salam',
    'assalam',
    'asalam',
    'waalaikumussalam',
    'mana',
    'sesudah',
    'berkenaan',
    'manakah',
    'satu',
    'dua',
    'tiga',
    'empat',
    'lima',
    'enam',
    'tujuh',
    'lapan',
    'sebilan',
    'sepuluh',
    'sebelas',
    'belas',
    'puluh',
    'ratus',
    'seratus',
    'seribu',
    'sejuta',
    'juta',
    'masa',
    'sesudahnya',
    'berupa',
    'manapun',
    'sesungguhnya',
    'beserta',
    'masih',
    'sesungguhnyakah',
    'biarpun',
    'masing',
    'guna',
    'setelah',
    'bila',
    'masing-masing',
    'setiap',
    'bilakah',
    'melainkan',
    'nk',
    'siapa',
    'meh',
    'bilamana',
    'memang',
    'siapakah',
    'lol',
    'macam',
    'tgh',
    'tengah',
    'atas',
    'bawah',
    'perghh',
    'pergh',
    'kiri',
    'buat',
    'leh',
    'kanan',
    'mcm',
    'bisa',
    'mempunyai',
    'sini',
    'boleh',
    'mendapat',
    'situ',
    'bukan',
    'mendapati',
    'situlah',
    'bukankah',
    'mendapatkan',
    'suatu',
    'as',
    'yg',
    'kpd',
    'bukanlah',
    'mengadakan',
    'sudah',
    'dahulu',
    'mengapa',
    'sudahkah',
    'dalam',
    'dlm',
    'mengapakah',
    'sungguh',
    'dalamnya',
    'mengenai',
    'sungguhpun',
    'dan',
    'menjadi',
    'supaya',
    'bole',
    'blum',
    'dapat',
    'menyebabkan',
    'tadinya',
    'dapati',
    'menyebabkannya',
    'tahukah',
    'dapatkah',
    'mereka',
    'jgn',
    'tak',
    'dapatlah',
    'kau',
    'merekalah',
    'tahun',
    'tanpa',
    'dari',
    'merekapun',
    'tanya',
    'daripada',
    'meskipun',
    'tanyakanlah',
    'daripadaku',
    'mu',
    'tapi',
    'daripadamu',
    'nescaya',
    'telah',
    'daripadanya',
    'niscaya',
    'tentang',
    'demi',
    'nya',
    'tentu',
    'demikian',
    'olah',
    'terdapat',
    'demikianlah',
    'oleh',
    'terhadap',
    'dengan',
    'orang',
    'terhadapmu',
    'dengannya',
    'pada',
    'dah',
    'den',
    'hoi',
    'huih',
    'jom',
    'joom',
    'tok',
    'bang',
    'abang',
    'kak',
    'kakak',
    'int',
    'jd',
    'atuk',
    'nenek',
    'ayah',
    'mak',
    'mama',
    'baba',
    'gg',
    'termasuk',
    'di',
    'padahal',
    'terpaksa',
    'dia',
    'padamu',
    'tertentu',
    'dialah',
    'padanya',
    'tetapi',
    'didapat',
    'paling',
    'tiada',
    'xkan',
    'xleh',
    'xtahu',
    'tahu',
    'helai',
    'ketul',
    'biar',
    'keb',
    'ust',
    'dorang',
    'ehh',
    'tetap',
    'tryna',
    'shit',
    'xda',
    'xde',
    'didapati',
    'para',
    'tiadakah',
    'dimanakah',
    'xkira',
    'bul',
    'pasti',
    'tiadalah',
    'engkau',
    'patut',
    'tiap',
    'engkaukah',
    'nak',
    'sahaja',
    'saje',
    'keje',
    'pastu',
    'pon',
    'pun',
    'keja',
    'patutkah',
    'tiap-tiap',
    'engkaulah',
    'per',
    'tidak',
    'disebabkan',
    'sebab',
    'engkaupun',
    'pergilah',
    'tidakkah',
    'hai',
    'perkara',
    'tidaklah',
    'hampir',
    'perkaranya',
    'turut',
    'hampir-hampir',
    'perlu',
    'bt',
    'bn',
    'tun',
    'haji',
    'untuk',
    'hanya',
    'pernah',
    "'s",
    'org',
    'untukmu',
    'hanyalah',
    'pertama',
    'wahai',
    'hendak',
    'woi',
    'wui',
    'pula',
    'hj',
    'photo',
    'video',
    'ir',
    'tuan',
    'yb',
    'puan',
    'walau',
    'prof',
    'hendaklah',
    'pun',
    'walaupun',
    'hingga',
    'sahaja',
    'keb',
    'bir',
    'tetap',
    'utk',
    'nah',
    'bar',
    'last',
    'ya',
    'tan',
    'wan',
    'sri',
    'dr',
    'ia',
    'sebaliknya',
    'adalah',
    'sahaja',
    'yaini',
    'iaitu',
    'saling',
    'yaitu',
    'ialah',
    'sama',
    'yakni',
    'ianya',
    'sama-sama',
    'yang',
    'inginkah',
    'samakah',
    'ini',
    'sambil',
    'beliau',
    'ada',
    'adalah',
    'adanya',
    'adapun',
    'agak',
    'agaknya',
    'agar',
    'akan',
    'akankah',
    'akhir',
    'akhiri',
    'akhirnya',
    'aku',
    'akulah',
    'amat',
    'amatlah',
    'anda',
    'andalah',
    'antar',
    'antara',
    'antaranya',
    'apa',
    'apaan',
    'apabila',
    'apakah',
    'apalagi',
    'apatah',
    'artinya',
    'asal',
    'asalkan',
    'atas',
    'atau',
    'ataukah',
    'ataupun',
    'awal',
    'awalnya',
    'bagai',
    'bagaikan',
    'bagaimana',
    'bagaimanakah',
    'bagaimanapun',
    'bagi',
    'bagian',
    'bahkan',
    'bahwa',
    'bahwasanya',
    'baik',
    'bakal',
    'bakalan',
    'balik',
    'banyak',
    'bapak',
    'baru',
    'bawah',
    'beberapa',
    'begini',
    'beginian',
    'beginikah',
    'beginilah',
    'begitu',
    'begitukah',
    'begitulah',
    'begitupun',
    'bekerja',
    'belakang',
    'belakangan',
    'belum',
    'belumlah',
    'benar',
    'benarkah',
    'benarlah',
    'berada',
    'berakhir',
    'berakhirlah',
    'berakhirnya',
    'berapa',
    'berapakah',
    'berapalah',
    'berapapun',
    'berarti',
    'berawal',
    'berbagai',
    'berdatangan',
    'beri',
    'berikan',
    'berikut',
    'berikutnya',
    'berjumlah',
    'berkali-kali',
    'berkata',
    'berkehendak',
    'berkeinginan',
    'berkenaan',
    'berlainan',
    'berlalu',
    'berlangsung',
    'berlebihan',
    'bermacam',
    'bermacam-macam',
    'bermaksud',
    'bermula',
    'bersama',
    'bersama-sama',
    'bersiap',
    'bersiap-siap',
    'bertanya',
    'bertanya-tanya',
    'berturut',
    'berturut-turut',
    'bertutur',
    'berujar',
    'berupa',
    'besar',
    'betul',
    'betulkah',
    'biasa',
    'biasanya',
    'bila',
    'bilakah',
    'bisa',
    'bisakah',
    'boleh',
    'bolehkah',
    'bolehlah',
    'buat',
    'bukan',
    'bukankah',
    'bukanlah',
    'bukannya',
    'bulan',
    'bung',
    'cara',
    'caranya',
    'cukup',
    'cukupkah',
    'cukuplah',
    'cuma',
    'dahulu',
    'dalam',
    'dan',
    'dapat',
    'dari',
    'daripada',
    'datang',
    'dekat',
    'demi',
    'demikian',
    'demikianlah',
    'dengan',
    'depan',
    'di',
    'dia',
    'diakhiri',
    'diakhirinya',
    'dialah',
    'diantara',
    'diantaranya',
    'diberi',
    'diberikan',
    'diberikannya',
    'dibuat',
    'dibuatnya',
    'didapat',
    'didatangkan',
    'digunakan',
    'diibaratkan',
    'diibaratkannya',
    'diingat',
    'diingatkan',
    'diinginkan',
    'dijawab',
    'dijelaskan',
    'dijelaskannya',
    'dikarenakan',
    'dikatakan',
    'dikatakannya',
    'dikerjakan',
    'diketahui',
    'diketahuinya',
    'dikira',
    'dilakukan',
    'dilalui',
    'dilihat',
    'dimaksud',
    'dimaksudkan',
    'dimaksudkannya',
    'dimaksudnya',
    'diminta',
    'dimintai',
    'dimisalkan',
    'dimulai',
    'dimulailah',
    'dimulainya',
    'dimungkinkan',
    'dini',
    'dipastikan',
    'diperbuat',
    'diperbuatnya',
    'dipergunakan',
    'diperkirakan',
    'diperlihatkan',
    'diperlukan',
    'diperlukannya',
    'dipersoalkan',
    'dipertanyakan',
    'dipunyai',
    'diri',
    'dirinya',
    'disampaikan',
    'disebut',
    'disebutkan',
    'disebutkannya',
    'disini',
    'disinilah',
    'ditambahkan',
    'ditandaskan',
    'ditanya',
    'ditanyai',
    'ditanyakan',
    'ditegaskan',
    'ditujukan',
    'ditunjuk',
    'ditunjuki',
    'ditunjukkan',
    'ditunjukkannya',
    'ditunjuknya',
    'dituturkan',
    'dituturkannya',
    'diucapkan',
    'diucapkannya',
    'diungkapkan',
    'dong',
    'dua',
    'dulu',
    'empat',
    'enggak',
    'enggaknya',
    'entah',
    'entahlah',
    'guna',
    'gunakan',
    'hal',
    'hampir',
    'hanya',
    'hanyalah',
    'hari',
    'harus',
    'haruslah',
    'harusnya',
    'hendak',
    'hendaklah',
    'hendaknya',
    'hingga',
    'ia',
    'ialah',
    'ibarat',
    'ibaratkan',
    'ibaratnya',
    'ibu',
    'ikut',
    'ingat',
    'ingat-ingat',
    'ingin',
    'inginkah',
    'inginkan',
    'ini',
    'inikah',
    'inilah',
    'itu',
    'itukah',
    'itulah',
    'jadi',
    'jadilah',
    'jadinya',
    'jangan',
    'jangankan',
    'janganlah',
    'jauh',
    'jawab',
    'jawaban',
    'jawabnya',
    'jelas',
    'jelaskan',
    'jelaslah',
    'jelasnya',
    'jika',
    'jikalau',
    'juga',
    'jumlah',
    'jumlahnya',
    'justru',
    'kala',
    'kalau',
    'kalaulah',
    'kalaupun',
    'kalian',
    'kami',
    'kamilah',
    'kamu',
    'kamulah',
    'kan',
    'kapan',
    'kapankah',
    'kapanpun',
    'karena',
    'karenanya',
    'kasus',
    'kata',
    'katakan',
    'katakanlah',
    'katanya',
    'ke',
    'keadaan',
    'kebetulan',
    'kecil',
    'kedua',
    'keduanya',
    'keinginan',
    'kelamaan',
    'kelihatan',
    'kelihatannya',
    'kelima',
    'keluar',
    'kembali',
    'kemudian',
    'kemungkinan',
    'kemungkinannya',
    'kenapa',
    'kepada',
    'kepadanya',
    'kesampaian',
    'keseluruhan',
    'keseluruhannya',
    'keterlaluan',
    'ketika',
    'khususnya',
    'kini',
    'kinilah',
    'kira',
    'kira-kira',
    'kiranya',
    'kita',
    'kitalah',
    'kok',
    'kurang',
    'lagi',
    'lagian',
    'lah',
    'lain',
    'lainnya',
    'lalu',
    'lama',
    'lamanya',
    'lanjut',
    'lanjutnya',
    'lebih',
    'lewat',
    'lima',
    'luar',
    'macam',
    'maka',
    'makanya',
    'makin',
    'malah',
    'malahan',
    'mampu',
    'mampukah',
    'mana',
    'manakala',
    'manalagi',
    'masa',
    'masalah',
    'masalahnya',
    'masih',
    'masihkah',
    'masing',
    'masing-masing',
    'mau',
    'maupun',
    'melainkan',
    'melakukan',
    'melalui',
    'melihat',
    'melihatnya',
    'memang',
    'memastikan',
    'memberi',
    'memberikan',
    'membuat',
    'memerlukan',
    'memihak',
    'meminta',
    'memintakan',
    'memisalkan',
    'memperbuat',
    'mempergunakan',
    'memperkirakan',
    'memperlihatkan',
    'mempersiapkan',
    'mempersoalkan',
    'mempertanyakan',
    'mempunyai',
    'memulai',
    'memungkinkan',
    'menaiki',
    'menambahkan',
    'menandaskan',
    'menanti',
    'menanti-nanti',
    'menantikan',
    'menanya',
    'menanyai',
    'menanyakan',
    'mendapat',
    'mendapatkan',
    'mendatang',
    'mendatangi',
    'mendatangkan',
    'menegaskan',
    'mengakhiri',
    'mengapa',
    'mengatakan',
    'mengatakannya',
    'mengenai',
    'mengerjakan',
    'mengetahui',
    'menggunakan',
    'menghendaki',
    'mengibaratkan',
    'mengibaratkannya',
    'mengingat',
    'mengingatkan',
    'menginginkan',
    'mengira',
    'mengucapkan',
    'mengucapkannya',
    'mengungkapkan',
    'menjadi',
    'menjawab',
    'menjelaskan',
    'menuju',
    'menunjuk',
    'menunjuki',
    'menunjukkan',
    'menunjuknya',
    'menurut',
    'menuturkan',
    'menyampaikan',
    'menyangkut',
    'menyatakan',
    'menyebutkan',
    'menyeluruh',
    'menyiapkan',
    'merasa',
    'mereka',
    'merekalah',
    'merupakan',
    'meski',
    'meskipun',
    'meyakini',
    'meyakinkan',
    'minta',
    'mirip',
    'misal',
    'misalkan',
    'misalnya',
    'mula',
    'mulai',
    'mulailah',
    'mulanya',
    'mungkin',
    'mungkinkah',
    'nah',
    'naik',
    'namun',
    'nanti',
    'nantinya',
    'nyaris',
    'nyatanya',
    'oleh',
    'olehnya',
    'pada',
    'padahal',
    'padanya',
    'pak',
    'paling',
    'panjang',
    'pantas',
    'para',
    'pasti',
    'pastilah',
    'penting',
    'pentingnya',
    'per',
    'percuma',
    'perlu',
    'perlukah',
    'perlunya',
    'pernah',
    'persoalan',
    'pertama',
    'pertama-tama',
    'pertanyaan',
    'pertanyakan',
    'pihak',
    'pihaknya',
    'pukul',
    'pula',
    'pun',
    'punya',
    'rasa',
    'rasanya',
    'rata',
    'rupanya',
    'saat',
    'saatnya',
    'sahaja',
    'sajalah',
    'saling',
    'sama',
    'sama-sama',
    'sambil',
    'sampai',
    'sampai-sampai',
    'sampaikan',
    'sana',
    'sangat',
    'sangatlah',
    'satu',
    'saya',
    'sayalah',
    'se',
    'sebab',
    'sebabnya',
    'sebagai',
    'sebagaimana',
    'sebagainya',
    'sebagian',
    'sebaik',
    'sebaik-baiknya',
    'sebaiknya',
    'sebaliknya',
    'sebanyak',
    'sebegini',
    'sebegitu',
    'sebelum',
    'sebelumnya',
    'sebenarnya',
    'seberapa',
    'sebesar',
    'sebetulnya',
    'sebisanya',
    'sebuah',
    'sebut',
    'sebutlah',
    'sebutnya',
    'secara',
    'secukupnya',
    'sedang',
    'sedangkan',
    'sedemikian',
    'sedikit',
    'sedikitnya',
    'seenaknya',
    'segala',
    'segalanya',
    'segera',
    'seharusnya',
    'sehingga',
    'seingat',
    'sejak',
    'sejauh',
    'sejenak',
    'sejumlah',
    'sekadar',
    'sekadarnya',
    'sekali',
    'sekali-kali',
    'sekalian',
    'sekaligus',
    'sekalipun',
    'sekarang',
    'sekarang',
    'sekecil',
    'seketika',
    'sekiranya',
    'sekitar',
    'sekitarnya',
    'sekurang-kurangnya',
    'sekurangnya',
    'sela',
    'selain',
    'selaku',
    'selalu',
    'selama',
    'selama-lamanya',
    'selamanya',
    'selanjutnya',
    'seluruh',
    'seluruhnya',
    'semacam',
    'semakin',
    'semampu',
    'semampunya',
    'semasa',
    'semasih',
    'semata',
    'semata-mata',
    'semaunya',
    'sementara',
    'semisal',
    'semisalnya',
    'sempat',
    'semua',
    'semuanya',
    'semula',
    'sendiri',
    'sendirian',
    'sendirinya',
    'seolah',
    'seolah-olah',
    'seorang',
    'sepanjang',
    'sepantasnya',
    'sepantasnyalah',
    'seperlunya',
    'seperti',
    'sepertinya',
    'sepihak',
    'sering',
    'seringnya',
    'serta',
    'serupa',
    'sesaat',
    'sesama',
    'sesampai',
    'sesegera',
    'sesekali',
    'seseorang',
    'sesuatu',
    'sesuatunya',
    'sesudah',
    'sesudahnya',
    'setelah',
    'setempat',
    'setengah',
    'seterusnya',
    'setiap',
    'setiba',
    'setibanya',
    'setidak-tidaknya',
    'setidaknya',
    'setinggi',
    'seusai',
    'sewaktu',
    'siap',
    'siapa',
    'siapakah',
    'siapapun',
    'sini',
    'sinilah',
    'soal',
    'soalnya',
    'suatu',
    'sudah',
    'sudahkah',
    'sudahlah',
    'supaya',
    'tadi',
    'tadinya',
    'tahu',
    'tahun',
    'tak',
    'tambah',
    'tambahnya',
    'tampak',
    'tampaknya',
    'tandas',
    'tandasnya',
    'tanpa',
    'tanya',
    'tanyakan',
    'tanyanya',
    'tapi',
    'tegas',
    'tegasnya',
    'telah',
    'tempat',
    'tengah',
    'tentang',
    'tentu',
    'tentulah',
    'tentunya',
    'tepat',
    'terakhir',
    'terasa',
    'terbanyak',
    'terdahulu',
    'terdapat',
    'terdiri',
    'terhadap',
    'terhadapnya',
    'teringat',
    'teringat-ingat',
    'terjadi',
    'terjadilah',
    'terjadinya',
    'terkira',
    'terlalu',
    'terlebih',
    'terlihat',
    'termasuk',
    'ternyata',
    'tersampaikan',
    'tersebut',
    'tersebutlah',
    'tertentu',
    'tertuju',
    'terus',
    'terutama',
    'tetap',
    'tetapi',
    'tiap',
    'tiba',
    'tiba-tiba',
    'tidak',
    'tidakkah',
    'tidaklah',
    'tiga',
    'tinggi',
    'toh',
    'tunjuk',
    'turut',
    'tutur',
    'tuturnya',
    'ucap',
    'ucapnya',
    'ujar',
    'ujarnya',
    'umum',
    'umumnya',
    'ungkap',
    'ungkapnya',
    'untuk',
    'usah',
    'usai',
    'waduh',
    'wah',
    'wahai',
    'waktu',
    'waktunya',
    'walau',
    'walaupun',
    'wong',
    'yaitu',
    'yakin',
    'yakni',
    'yang',
]

group_compound = [
    'ng',
    'nt',
    'mp',
    'gs',
    'rb',
    'th',
    'st',
    'mb',
    'ny',
    'gk',
    'ky',
    'km',
    'gd',
    'nd',
]

"""
mw = []
for w in malaya.texts._malay_words._malay_words:
    mw.append(''.join([v for v in w if v in vowels]))
subwords = []
for w in mw:
    for i in range(0, len(w), 4):
        subwords.append(w[i: i + 4])
subwords = [w for w in subwords if len(w) == 4]
u, c = np.unique(subwords, return_counts = True)
quad_vowels = u
"""

quad_vowels = [
    'aaaa',
    'aaai',
    'aaao',
    'aaau',
    'aaea',
    'aaee',
    'aaei',
    'aaeo',
    'aaeu',
    'aaia',
    'aaie',
    'aaii',
    'aaio',
    'aaiu',
    'aaoa',
    'aaoe',
    'aaoi',
    'aaoo',
    'aaou',
    'aaua',
    'aaue',
    'aaui',
    'aauu',
    'aeaa',
    'aeae',
    'aeai',
    'aeei',
    'aeia',
    'aeie',
    'aeii',
    'aeio',
    'aeiu',
    'aeoa',
    'aeoe',
    'aeoi',
    'aeoo',
    'aeua',
    'aeui',
    'aiaa',
    'aiae',
    'aiai',
    'aiao',
    'aiau',
    'aiea',
    'aiee',
    'aiei',
    'aieo',
    'aieu',
    'aiia',
    'aiie',
    'aiii',
    'aiio',
    'aiiu',
    'aioa',
    'aioe',
    'aioi',
    'aioo',
    'aiou',
    'aiua',
    'aiui',
    'aiuu',
    'aoaa',
    'aoai',
    'aoao',
    'aoau',
    'aoea',
    'aoee',
    'aoei',
    'aoeo',
    'aoeu',
    'aoia',
    'aoie',
    'aoii',
    'aoio',
    'aoiu',
    'aooa',
    'aooe',
    'aooi',
    'aooo',
    'aoou',
    'aoui',
    'auaa',
    'auae',
    'auai',
    'auau',
    'auea',
    'auei',
    'aueo',
    'aueu',
    'auia',
    'auie',
    'auii',
    'auio',
    'auiu',
    'auoa',
    'auoe',
    'auoi',
    'auoo',
    'auua',
    'auui',
    'auuu',
    'eaaa',
    'eaae',
    'eaai',
    'eaao',
    'eaau',
    'eaea',
    'eaee',
    'eaei',
    'eaeo',
    'eaeu',
    'eaia',
    'eaie',
    'eaii',
    'eaio',
    'eaiu',
    'eaoa',
    'eaoi',
    'eaoo',
    'eaua',
    'eaue',
    'eaui',
    'eauu',
    'eeaa',
    'eeae',
    'eeai',
    'eeao',
    'eeau',
    'eeea',
    'eeee',
    'eeei',
    'eeeo',
    'eeeu',
    'eeia',
    'eeie',
    'eeii',
    'eeiu',
    'eeoa',
    'eeoe',
    'eeoi',
    'eeoo',
    'eeou',
    'eeua',
    'eeui',
    'eeuu',
    'eiaa',
    'eiae',
    'eiai',
    'eiao',
    'eiau',
    'eiea',
    'eiee',
    'eiei',
    'eiia',
    'eiie',
    'eiii',
    'eiio',
    'eiiu',
    'eioa',
    'eioe',
    'eioi',
    'eioo',
    'eiou',
    'eiua',
    'eiui',
    'eiuu',
    'eoaa',
    'eoae',
    'eoai',
    'eoau',
    'eoea',
    'eoee',
    'eoei',
    'eoeo',
    'eoeu',
    'eoia',
    'eoie',
    'eoii',
    'eoio',
    'eoiu',
    'eooa',
    'eooe',
    'eooi',
    'eooo',
    'eoua',
    'eoue',
    'eoui',
    'euaa',
    'euae',
    'euai',
    'euau',
    'euea',
    'euei',
    'eueo',
    'euia',
    'euii',
    'euio',
    'euoa',
    'euoi',
    'euoo',
    'euua',
    'euue',
    'euui',
    'euuu',
    'iaaa',
    'iaai',
    'iaau',
    'iaea',
    'iaee',
    'iaei',
    'iaia',
    'iaie',
    'iaii',
    'iaiu',
    'iaoa',
    'iaoi',
    'iaoo',
    'iaua',
    'iaue',
    'iaui',
    'iauu',
    'ieaa',
    'ieai',
    'ieao',
    'ieau',
    'ieea',
    'ieee',
    'ieei',
    'ieeo',
    'ieeu',
    'ieia',
    'ieie',
    'ieii',
    'ieio',
    'ieiu',
    'ieoa',
    'ieoe',
    'ieoi',
    'ieoo',
    'ieua',
    'ieui',
    'ieuu',
    'iiaa',
    'iiai',
    'iiao',
    'iiau',
    'iiea',
    'iiee',
    'iieu',
    'iiia',
    'iiie',
    'iiii',
    'iiiu',
    'iioa',
    'iioe',
    'iioo',
    'iiua',
    'iiui',
    'ioaa',
    'ioai',
    'ioao',
    'ioau',
    'ioea',
    'ioee',
    'ioei',
    'ioeo',
    'ioeu',
    'ioia',
    'ioie',
    'ioii',
    'ioio',
    'ioiu',
    'iooa',
    'iooe',
    'iooi',
    'ioua',
    'ioue',
    'iuaa',
    'iuae',
    'iuai',
    'iuao',
    'iuau',
    'iuea',
    'iuia',
    'iuii',
    'iuiu',
    'iuoa',
    'iuua',
    'iuui',
    'oaaa',
    'oaai',
    'oaea',
    'oaee',
    'oaei',
    'oaia',
    'oaie',
    'oaii',
    'oaio',
    'oaiu',
    'oaoa',
    'oaoe',
    'oaoi',
    'oaua',
    'oeae',
    'oeai',
    'oeao',
    'oeea',
    'oeei',
    'oeeo',
    'oeia',
    'oeii',
    'oeio',
    'oeiu',
    'oeoa',
    'oeoe',
    'oeua',
    'oiaa',
    'oiai',
    'oiao',
    'oiau',
    'oiea',
    'oiei',
    'oieo',
    'oiia',
    'oiie',
    'oiii',
    'oiiu',
    'oioe',
    'oioi',
    'oioo',
    'oiou',
    'oiua',
    'oiue',
    'oiui',
    'oiuu',
    'ooaa',
    'ooai',
    'ooau',
    'ooea',
    'ooee',
    'ooei',
    'ooeu',
    'ooia',
    'ooie',
    'ooii',
    'ooio',
    'ooiu',
    'oooa',
    'oooi',
    'oooo',
    'ooui',
    'oouu',
    'ouaa',
    'ouai',
    'ouea',
    'ouia',
    'ouie',
    'ouii',
    'ouiu',
    'uaaa',
    'uaai',
    'uaau',
    'uaea',
    'uaei',
    'uaeu',
    'uaia',
    'uaie',
    'uaii',
    'uaiu',
    'uaoa',
    'uaoe',
    'uaoi',
    'uaua',
    'uaui',
    'ueaa',
    'ueai',
    'ueee',
    'ueeo',
    'ueia',
    'ueie',
    'ueoi',
    'uiaa',
    'uiae',
    'uiai',
    'uiau',
    'uiea',
    'uiei',
    'uiia',
    'uiii',
    'uioa',
    'uioe',
    'uiue',
    'uiui',
    'uiuu',
    'uoao',
    'uoia',
    'uoiu',
    'uooa',
    'uuaa',
    'uuae',
    'uuai',
    'uuau',
    'uuea',
    'uuei',
    'uueo',
    'uueu',
    'uuia',
    'uuie',
    'uuii',
    'uuio',
    'uuoa',
    'uuoe',
    'uuua',
    'uuui',
    'uuuu',
]



import itertools
import random
import inspect
import numpy as np
import string as string_function
from .preprocessing import _tokenizer
from tensorflow.keras.preprocessing.sequence import pad_sequences

_accepted_pos = [
    'ADJ',
    'ADP',
    'ADV',
    'ADX',
    'CCONJ',
    'DET',
    'NOUN',
    'NUM',
    'PART',
    'PRON',
    'PROPN',
    'SCONJ',
    'SYM',
    'VERB',
    'X',
]
_accepted_entities = [
    'OTHER',
    'law',
    'location',
    'organization',
    'person',
    'quantity',
    'time',
    'event',
]


def _check_digit(string):
    return any(i.isdigit() for i in string)


def _make_upper(p, o):
    p_split = p.split()
    o_split = o.split()
    return ' '.join(
        [
            s.title() if o_split[no][0].isupper() else s
            for no, s in enumerate(p_split)
        ]
    )


def _simple_textcleaning(string, lowering = True):
    """
    use by topic modelling
    only accept A-Z, a-z
    """
    string = unidecode(string)
    string = re.sub(
        '[^A-Za-z0-9\-\\/\'"!@#$%^&*()-+={}\[\];:<>,.? ]+', ' ', string
    )
    return re.sub(r'[ ]+', ' ', string.lower() if lowering else string).strip()


def _pad_sequence(
    sequence,
    n,
    pad_left = False,
    pad_right = False,
    left_pad_symbol = None,
    right_pad_symbol = None,
):
    sequence = iter(sequence)
    if pad_left:
        sequence = itertools.chain((left_pad_symbol,) * (n - 1), sequence)
    if pad_right:
        sequence = itertools.chain(sequence, (right_pad_symbol,) * (n - 1))
    return sequence


def to_ids(string, tokenizer):
    words = []
    for no, word in enumerate(string):
        if word == '<mask>':
            words.append(word)
        else:
            words.extend(tokenizer.tokenize(word))
    masked_tokens = ['<cls>'] + words + ['<sep>']
    masked_ids = tokenizer.convert_tokens_to_ids(masked_tokens)
    return masked_ids, masked_ids.index(6)


def ngrams(
    sequence,
    n,
    pad_left = False,
    pad_right = False,
    left_pad_symbol = None,
    right_pad_symbol = None,
):
    """
    generate ngrams.

    Parameters
    ----------
    sequence : list of str
        list of tokenize words.
    n : int
        ngram size

    Returns
    -------
    ngram: list
    """
    sequence = _pad_sequence(
        sequence, n, pad_left, pad_right, left_pad_symbol, right_pad_symbol
    )

    history = []
    while n > 1:
        try:
            next_item = next(sequence)
        except StopIteration:
            return
        history.append(next_item)
        n -= 1
    for item in sequence:
        history.append(item)
        yield tuple(history)
        del history[0]


def pos_entities_ngram(
    result_pos,
    result_entities,
    ngram = (1, 3),
    accept_pos = ['NOUN', 'PROPN', 'VERB'],
    accept_entities = ['law', 'location', 'organization', 'person', 'time'],
):
    """
    generate ngrams.

    Parameters
    ----------
    result_pos : list of tuple
        result from POS recognition.
    result_entities : list of tuple
        result of Entities recognition.
    ngram : tuple
        ngram sizes.
    accept_pos : list of str
        accepted POS elements.
    accept_entities : list of str
        accept entities elements.

    Returns
    -------
    result: list
    """
    if not isinstance(result_pos, list):
        raise ValueError('result_pos must be a list')
    if not isinstance(result_pos[0], tuple):
        raise ValueError('result_pos must be a list of tuple')
    if not isinstance(ngram, tuple):
        raise ValueError('ngram must be a tuple')
    if not len(ngram) == 2:
        raise ValueError('ngram size must equal to 2')
    if not isinstance(accept_pos, list):
        raise ValueError('accept_pos must be a list')
    if not isinstance(accept_entities, list):
        raise ValueError('accept_entites must be a list')
    if not all([i in _accepted_pos for i in accept_pos]):
        raise ValueError(
            'accept_pos must be a subset or equal of supported POS, please run malaya.describe_pos() to get supported POS'
        )
    if not all([i in _accepted_entities for i in accept_entities]):
        raise ValueError(
            'accept_entites must be a subset or equal of supported entities, please run malaya.describe_entities() to get supported entities'
        )

    words = []
    sentences = []
    for no in range(len(result_pos)):
        if (
            result_pos[no][1] in accept_pos
            or result_entities[no][1] in accept_entities
        ):
            words.append(result_pos[no][0])
    for gram in range(ngram[0], ngram[1] + 1, 1):
        gram_words = list(ngrams(words, gram))
        for sentence in gram_words:
            sentences.append(' '.join(sentence))
    return list(set(sentences))


def sentence_ngram(sentence, ngram = (1, 3)):
    """
    generate ngram for a text

    Parameters
    ----------
    sentence: str
    ngram : tuple
        ngram sizes.

    Returns
    -------
    result: list
    """
    if not isinstance(sentence, str):
        raise ValueError('sentence must be a string')
    if not isinstance(ngram, tuple):
        raise ValueError('ngram must be a tuple')
    if not len(ngram) == 2:
        raise ValueError('ngram size must equal to 2')

    words = sentence.split()
    sentences = []
    for gram in range(ngram[0], ngram[1] + 1, 1):
        gram_words = list(ngrams(words, gram))
        for sentence in gram_words:
            sentences.append(' '.join(sentence))
    return list(set(sentences))


def wordvector_augmentation(
    string,
    wordvector,
    threshold = 0.5,
    top_n = 5,
    soft = False,
    cleaning_function = None,
):
    """
    augmenting a string using wordvector.

    Parameters
    ----------
    string: str
    wordvector: object
        wordvector interface object.
    threshold: float, optional (default=0.5)
        random selection for a word.
    soft: bool, optional (default=False)
        if True, a word not in the dictionary will be replaced with nearest jarowrinkler ratio.
        if False, it will throw an exception if a word not in the dictionary.
    top_n: int, (default=5)
        number of nearest neighbors returned.
    cleaning_function: function, (default=None)
        function to clean text.

    Returns
    -------
    result: list
    """
    if not isinstance(string, str):
        raise ValueError('string must be a string')
    if not isinstance(threshold, float):
        raise ValueError('threshold must be a float')
    if not (threshold > 0 and threshold < 1):
        raise ValueError('threshold must be bigger than 0 and less than 1')
    if not isinstance(soft, bool):
        raise ValueError('soft must be a boolean')
    if not hasattr(wordvector, 'batch_n_closest'):
        raise ValueError('wordvector must has `batch_n_closest` method')
    if not hasattr(wordvector, '_dictionary'):
        raise ValueError('wordvector must has `_dictionary` attribute')
    if not isinstance(top_n, int):
        raise ValueError('top_n must be an integer')

    original_string = string
    if cleaning_function:
        string = cleaning_function(string)
    string = _tokenizer(string)
    original_string = string[:]
    selected = []
    for no, w in enumerate(string):
        if w in string_function.punctuation:
            continue
        if w[0].isupper():
            continue
        if random.random() > threshold:
            selected.append((no, w))

    if not len(selected):
        raise ValueError(
            'no words can augmented, make sure words available are not punctuation or proper nouns.'
        )

    indices, words = [i[0] for i in selected], [i[1] for i in selected]
    batch_parameters = list(
        inspect.signature(wordvector.batch_n_closest).parameters.keys()
    )
    if 'soft' in batch_parameters:
        results = wordvector.batch_n_closest(
            words, num_closest = top_n, soft = soft
        )
    else:
        results = wordvector.batch_n_closest(words, num_closest = top_n)

    augmented = []
    for i in range(top_n):
        string_ = string[:]
        for no in range(len(results)):
            string_[indices[no]] = results[no][i]
        augmented.append(
            _make_upper(' '.join(string_), ' '.join(original_string))
        )
    return augmented


def transformer_augmentation(
    string,
    model,
    threshold = 0.5,
    top_p = 0.8,
    top_k = 100,
    temperature = 0.8,
    top_n = 5,
    cleaning_function = None,
):

    """
    augmenting a string using transformer + nucleus sampling / top-k sampling.

    Parameters
    ----------
    string: str
    model: object
        transformer interface object. Right now only supported BERT.
    threshold: float, optional (default=0.5)
        random selection for a word.
    top_p: float, optional (default=0.8)
        cumulative sum of probabilities to sample a word. If top_n bigger than 0, the model will use nucleus sampling, else top-k sampling.
    top_k: int, optional (default=100)
        k for top-k sampling.
    temperature: float, optional (default=0.8)
        logits * temperature.
    top_n: int, (default=5)
        number of nearest neighbors returned.
    cleaning_function: function, (default=None)
        function to clean text.

    Returns
    -------
    result: list
    """

    if not isinstance(string, str):
        raise ValueError('string must be a string')
    if not hasattr(model, 'samples'):
        raise ValueError('model must has `samples` attribute')
    if not isinstance(threshold, float):
        raise ValueError('threshold must be a float')
    if not (threshold > 0 and threshold < 1):
        raise ValueError('threshold must be bigger than 0 and less than 1')
    if not isinstance(top_p, float):
        raise ValueError('top_p must be a float')
    if not top_p > 0:
        raise ValueError('top_p must be bigger than 0')
    if not isinstance(top_k, int):
        raise ValueError('top_k must be an integer')
    if not top_k > 0:
        raise ValueError('top_k must be bigger than 0')
    if not isinstance(temperature, float):
        raise ValueError('temperature must be a float')
    if not (temperature > 0 and threshold < 1):
        raise ValueError('temperature must be bigger than 0 and less than 1')
    if not isinstance(top_n, int):
        raise ValueError('top_n must be an integer')
    if not top_n > 0:
        raise ValueError('top_n must be bigger than 0')
    if top_n > top_k:
        raise ValueError('top_k must be bigger than top_n')

    original_string = string
    if cleaning_function:
        string = cleaning_function(string)
    string = _tokenizer(string)
    results = []
    for token_idx, token in enumerate(string):
        if token in string_function.punctuation:
            continue
        if token[0].isupper():
            continue
        if token.isdigit():
            continue
        if random.random() > threshold:
            results.append(token_idx)

    if not len(results):
        raise ValueError(
            'no words can augmented, make sure words available are not punctuation or proper nouns.'
        )

    maskeds, indices = [], []
    for index in results:
        new = string[:]
        new[index] = '<mask>'
        mask, ind = to_ids(new, model._tokenizer)
        maskeds.append(mask)
        indices.append(ind)

    masked_padded = pad_sequences(maskeds, padding = 'post')
    batch_indices = np.array([np.arange(len(indices)), indices]).T
    samples = model._sess.run(
        model.samples,
        feed_dict = {
            model.X: masked_padded,
            model.top_p: top_p,
            model.top_k: top_k,
            model.temperature: temperature,
            model.indices: batch_indices,
            model.k: top_n,
        },
    )
    outputs = []
    for i in range(samples.shape[1]):
        sample_i = samples[:, i]
        samples_tokens = model._tokenizer.convert_ids_to_tokens(
            sample_i.tolist()
        )
        new_splitted = ['▁' + w if len(w) > 1 else w for w in string]
        for no, index in enumerate(results):
            new_splitted[index] = samples_tokens[no]
        new = ''.join(model._tokenizer.sp_model.DecodePieces(new_splitted))
        outputs.append(new)
    return outputs



import tensorflow as tf
from create_pretraining_data import *
import json
import re
import sentencepiece as spm
from prepro_utils import preprocess_text, encode_ids, encode_pieces

sp_model = spm.SentencePieceProcessor()
sp_model.Load('sp10m.cased.v8.model')

with open('sp10m.cased.v8.vocab') as fopen:
    v = fopen.read().split('\n')[:-1]
v = [i.split('\t') for i in v]
v = {i[0]: i[1] for i in v}


class Tokenizer:
    def __init__(self, v):
        self.vocab = v
        pass

    def tokenize(self, string):
        return encode_pieces(
            sp_model, string, return_unicode = False, sample = False
        )

    def convert_tokens_to_ids(self, tokens):
        return [sp_model.PieceToId(piece) for piece in tokens]

    def convert_ids_to_tokens(self, ids):
        return [sp_model.IdToPiece(i) for i in ids]


tokenizer = Tokenizer(v)
files = '../dumping-all.txt'.split(',')

print('Reading from input files', files)

output_files = ('bert-combined.tfrecord').split(',')
print('Output filename', output_files)

input_files = []
for input_pattern in files:
    input_files.extend(tf.gfile.Glob(input_pattern))

tf.logging.info('*** Reading from input files ***')
for input_file in input_files:
    tf.logging.info('  %s', input_file)

import random

max_seq_length = 128
dupe_factor = 5
max_predictions_per_seq = 20
masked_lm_prob = 0.15
short_seq_prob = 0.1
rng = random.Random(12345)
instances = create_training_instances(
    input_files,
    tokenizer,
    max_seq_length,
    dupe_factor,
    short_seq_prob,
    masked_lm_prob,
    max_predictions_per_seq,
    rng,
)

tf.logging.info('*** Writing to output files ***')
for output_file in output_files:
    tf.logging.info('  %s', output_file)

write_instance_to_example_files(
    instances, tokenizer, max_seq_length, max_predictions_per_seq, output_files
)



import numpy as np
from ..texts._text_functions import (
    simple_textcleaning,
    classification_textcleaning,
    entities_textcleaning,
    language_detection_textcleaning,
    tag_chunk,
)
from .._utils._utils import add_neutral as neutral


class BAYES:
    def __init__(
        self, multinomial, label, vectorize, cleaning = simple_textcleaning
    ):
        self._multinomial = multinomial
        self._label = label
        self._vectorize = vectorize
        self._cleaning = cleaning


class BINARY_BAYES(BAYES):
    def __init__(
        self, multinomial, label, vectorize, cleaning = simple_textcleaning
    ):
        BAYES.__init__(self, multinomial, label, vectorize, cleaning)

    def predict(self, string, get_proba = False, add_neutral = True):
        """
        Classify a string.

        Parameters
        ----------
        string : str
        get_proba: bool, optional (default=False)
            If True, it will return probability of classes.
        add_neutral: bool, optional (default=True)
            if True, it will add neutral probability.

        Returns
        -------
        string: result
        """
        if not isinstance(string, str):
            raise ValueError('input must be a string')
        if not isinstance(get_proba, bool):
            raise ValueError('get_proba must be a boolean')
        if not isinstance(add_neutral, bool):
            raise ValueError('add_neutral must be a boolean')

        if add_neutral:
            label = self._label + ['neutral']
        else:
            label = self._label
        vectors = self._vectorize.transform([self._cleaning(string)])
        result = self._multinomial.predict_proba(vectors)
        if add_neutral:
            result = neutral(result)
        result = result[0]
        if get_proba:
            return {label[i]: result[i] for i in range(len(result))}
        else:
            return label[np.argmax(result)]

    def predict_batch(self, strings, get_proba = False, add_neutral = True):
        """
        Classify a list of strings.

        Parameters
        ----------
        strings: list
        get_proba: bool, optional (default=False)
            If True, it will return probability of classes.
        add_neutral: bool, optional (default=True)
            if True, it will add neutral probability.

        Returns
        -------
        string: list of results
        """
        if not isinstance(strings, list):
            raise ValueError('input must be a list')
        if not isinstance(strings[0], str):
            raise ValueError('input must be list of strings')
        if not isinstance(get_proba, bool):
            raise ValueError('get_proba must be a boolean')
        if not isinstance(add_neutral, bool):
            raise ValueError('add_neutral must be a boolean')

        if add_neutral:
            label = self._label + ['neutral']
        else:
            label = self._label

        strings = [self._cleaning(string) for string in strings]
        vectors = self._vectorize.transform(strings)
        results = self._multinomial.predict_proba(vectors)

        if add_neutral:
            results = neutral(results)

        if get_proba:
            outputs = []
            for result in results:
                outputs.append(
                    {label[i]: result[i] for i in range(len(result))}
                )
            return outputs
        else:
            return [label[result] for result in np.argmax(results, axis = 1)]


class MULTICLASS_BAYES(BAYES):
    def __init__(
        self, multinomial, label, vectorize, cleaning = simple_textcleaning
    ):
        BAYES.__init__(self, multinomial, label, vectorize, cleaning)

    def predict(self, string, get_proba = False):
        """
        Classify a string.

        Parameters
        ----------
        string : str
        get_proba: bool, optional (default=False)
            If True, it will return probability of classes.

        Returns
        -------
        string: result
        """
        if not isinstance(string, str):
            raise ValueError('input must be a string')
        if not isinstance(get_proba, bool):
            raise ValueError('get_proba must be a boolean')
        vectors = self._vectorize.transform([self._cleaning(string)])
        result = self._multinomial.predict_proba(vectors)[0]
        if get_proba:
            return {self._label[i]: result[i] for i in range(len(result))}
        else:
            return self._label[np.argmax(result)]

    def predict_batch(self, strings, get_proba = False):
        """
        Classify a list of strings.

        Parameters
        ----------
        strings: list
        get_proba: bool, optional (default=False)
            If True, it will return probability of classes.

        Returns
        -------
        string: list of results
        """
        if not isinstance(strings, list):
            raise ValueError('input must be a list')
        if not isinstance(strings[0], str):
            raise ValueError('input must be list of strings')
        if not isinstance(get_proba, bool):
            raise ValueError('get_proba must be a boolean')
        strings = [self._cleaning(string) for string in strings]
        vectors = self._vectorize.transform(strings)
        results = self._multinomial.predict_proba(vectors)
        if get_proba:
            outputs = []
            for result in results:
                outputs.append(
                    {self._label[i]: result[i] for i in range(len(result))}
                )
            return outputs
        else:
            return [
                self._label[result] for result in np.argmax(results, axis = 1)
            ]


class MULTILABEL_BAYES:
    def __init__(self, models, vectors, cleaning = simple_textcleaning):
        self._multinomial = models
        self._vectorize = vectors
        self._class_names = [
            'toxic',
            'severe_toxic',
            'obscene',
            'threat',
            'insult',
            'identity_hate',
        ]
        self._cleaning = cleaning

    def predict(self, string, get_proba = False):
        """
        Classify a string.

        Parameters
        ----------
        string : str
        get_proba: bool, optional (default=False)
            If True, it will return probability of classes.

        Returns
        -------
        string: result
        """
        if not isinstance(string, str):
            raise ValueError('input must be a string')
        vectors = self._vectorize.transform([self._cleaning(string)])
        result = self._multinomial.predict_proba(vectors)[0]
        arounded = np.around(result)
        results = {} if get_proba else []
        for no, label in enumerate(self._class_names):
            if get_proba:
                results[label] = result[no]
            else:
                prob = arounded[no]
                if prob:
                    results.append(label)
        return results

    def predict_batch(self, strings, get_proba = False):
        """
        Classify a list of strings.

        Parameters
        ----------
        strings: list
        get_proba: bool, optional (default=False)
            If True, it will return probability of classes.

        Returns
        -------
        string: list of results
        """
        if not isinstance(strings, list):
            raise ValueError('input must be a list')
        if not isinstance(strings[0], str):
            raise ValueError('input must be list of strings')

        strings = [self._cleaning(string) for string in strings]
        vectors = self._vectorize.transform(strings)
        result = self._multinomial.predict_proba(vectors)
        arounded = np.around(result)

        results = []
        for i, row in enumerate(result):
            nested_results = {} if get_proba else []
            for no, label in enumerate(self._class_names):
                if get_proba:
                    nested_results[label] = row[no]
                else:
                    prob = arounded[i, no]
                    if prob:
                        nested_results.append(label)
            results.append(nested_results)
        return results


class LANGUAGE_DETECTION:
    def __init__(self, model, label, vectorizer, mode = 'sklearn'):
        self._model = model
        self._label = label
        self._vectorizer = vectorizer
        self._mode = mode

    def predict(self, string, get_proba = False):
        """
        Classify a string.

        Parameters
        ----------
        string : str
        get_proba: bool, optional (default=False)
            If True, it will return probability of classes.

        Returns
        -------
        string: result
        """
        if not isinstance(string, str):
            raise ValueError('input must be a string')
        string = language_detection_textcleaning(string)
        vectors = self._vectorizer.transform([string])
        if get_proba:
            result = self._model.predict_proba(vectors)[0]
            return {self._label[i]: result[i] for i in range(len(result))}
        else:
            return self._label[self._model.predict(vectors)[0]]

    def predict_batch(self, strings, get_proba = False):
        """
        Classify a list of strings.

        Parameters
        ----------
        strings: list
        get_proba: bool, optional (default=False)
            If True, it will return probability of classes.

        Returns
        -------
        string: list of results
        """
        if not isinstance(strings, list):
            raise ValueError('input must be a list')
        if not isinstance(strings[0], str):
            raise ValueError('input must be list of strings')
        strings = [
            language_detection_textcleaning(string) for string in strings
        ]
        vectors = self._vectorizer.transform(strings)

        if get_proba:
            results = self._model.predict_proba(vectors)
            outputs = []
            for result in results:
                outputs.append(
                    {self._label[i]: result[i] for i in range(len(result))}
                )
            return outputs
        else:
            return [
                self._label[result] for result in self._model.predict(vectors)
            ]



from ._utils import _tag_class
from ._utils._paths import PATH_ENTITIES, S3_PATH_ENTITIES
from .texts._entity import _Entity_regex


_availability = {
    'bert': ['base', 'small'],
    'xlnet': ['base'],
    'albert': ['base'],
}


def available_transformer_model():
    """
    List available transformer Entity Tagging models.
    """
    return _availability


def transformer(model = 'xlnet', size = 'base', validate = True):
    """
    Load Transformer Entity Tagging model, transfer learning Transformer + CRF.

    Parameters
    ----------
    model : str, optional (default='bert')
        Model architecture supported. Allowed values:

        * ``'bert'`` - BERT architecture from google.
        * ``'xlnet'`` - XLNET architecture from google.
        * ``'albert'`` - ALBERT architecture from google.
    size : str, optional (default='base')
        Model size supported. Allowed values:

        * ``'base'`` - BASE size.
        * ``'small'`` - SMALL size.
    validate: bool, optional (default=True)
        if True, malaya will check model availability and download if not available.

    Returns
    -------
    MODEL : Transformer class
    """
    if not isinstance(model, str):
        raise ValueError('model must be a string')
    if not isinstance(size, str):
        raise ValueError('size must be a string')
    if not isinstance(validate, bool):
        raise ValueError('validate must be a boolean')

    model = model.lower()
    size = size.lower()
    if model not in _availability:
        raise Exception(
            'model not supported, please check supported models from malaya.entity.available_transformer_model()'
        )
    if size not in _availability[model]:
        raise Exception(
            'size not supported, please check supported models from malaya.entity.available_transformer_model()'
        )
    return _tag_class.transformer(
        PATH_ENTITIES,
        S3_PATH_ENTITIES,
        'entity',
        model = model,
        size = size,
        validate = validate,
    )


def general_entity(model = None):
    """
    Load Regex based general entities tagging along with another supervised entity tagging model.

    Parameters
    ----------
    model : object
        model must has `predict` method. Make sure the `predict` method returned [(string, label), (string, label)].

    Returns
    -------
    _Entity_regex: malaya.texts._entity._Entity_regex class
    """
    if not hasattr(model, 'predict') and model is not None:
        raise ValueError('model must has `predict` method')
    return _Entity_regex(model = model)



# coding=utf-8
# Copyright 2019 The Google Research Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python2, python3
"""Run masked LM/next sentence masked_lm pre-training for ALBERT."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import os
import time

os.environ[
    'CUDA_VISIBLE_DEVICES'
] = '1,2,3'  # I want to use second GPU and above

from six.moves import range
import tensorflow as tf

from albert import modeling
import custom_optimization as optimization
from tensorflow.python.distribute.cross_device_ops import (
    AllReduceCrossDeviceOps,
)
from tensorflow.python.estimator.run_config import RunConfig
from tensorflow.python.estimator.estimator import Estimator
from tensorflow.contrib import cluster_resolver as contrib_cluster_resolver
from tensorflow.contrib import data as contrib_data
from tensorflow.contrib import tpu as contrib_tpu

flags = tf.flags

FLAGS = flags.FLAGS

## Required parameters
flags.DEFINE_string(
    'albert_config_file',
    None,
    'The config json file corresponding to the pre-trained ALBERT model. '
    'This specifies the model architecture.',
)

flags.DEFINE_string(
    'input_file',
    None,
    'Input TF example files (can be a glob or comma separated).',
)

flags.DEFINE_string(
    'output_dir',
    None,
    'The output directory where the model checkpoints will be written.',
)

flags.DEFINE_string(
    'export_dir',
    None,
    'The output directory where the saved models will be written.',
)
## Other parameters
flags.DEFINE_string(
    'init_checkpoint',
    None,
    'Initial checkpoint (usually from a pre-trained ALBERT model).',
)

flags.DEFINE_integer(
    'max_seq_length',
    512,
    'The maximum total input sequence length after WordPiece tokenization. '
    'Sequences longer than this will be truncated, and sequences shorter '
    'than this will be padded. Must match data generation.',
)

flags.DEFINE_integer(
    'max_predictions_per_seq',
    20,
    'Maximum number of masked LM predictions per sequence. '
    'Must match data generation.',
)

flags.DEFINE_bool('do_train', True, 'Whether to run training.')

flags.DEFINE_bool('do_eval', False, 'Whether to run eval on the dev set.')

flags.DEFINE_integer('train_batch_size', 4096, 'Total batch size for training.')

flags.DEFINE_integer('eval_batch_size', 64, 'Total batch size for eval.')

flags.DEFINE_enum(
    'optimizer', 'lamb', ['adamw', 'lamb'], 'The optimizer for training.'
)

flags.DEFINE_float('learning_rate', 0.00176, 'The initial learning rate.')

flags.DEFINE_float('poly_power', 1.0, 'The power of poly decay.')

flags.DEFINE_integer('num_train_steps', 125000, 'Number of training steps.')

flags.DEFINE_integer('num_warmup_steps', 3125, 'Number of warmup steps.')

flags.DEFINE_integer('start_warmup_step', 0, 'The starting step of warmup.')

flags.DEFINE_integer(
    'save_checkpoints_steps', 5000, 'How often to save the model checkpoint.'
)

flags.DEFINE_integer(
    'iterations_per_loop',
    1000,
    'How many steps to make in each estimator call.',
)

flags.DEFINE_integer('max_eval_steps', 100, 'Maximum number of eval steps.')

flags.DEFINE_bool('use_tpu', False, 'Whether to use TPU or GPU/CPU.')

flags.DEFINE_bool(
    'init_from_group0',
    False,
    'Whether to initialize' 'parameters of other groups from group 0',
)

tf.flags.DEFINE_string(
    'tpu_name',
    None,
    'The Cloud TPU to use for training. This should be either the name '
    'used when creating the Cloud TPU, or a grpc://ip.address.of.tpu:8470 '
    'url.',
)

tf.flags.DEFINE_string(
    'tpu_zone',
    None,
    '[Optional] GCE zone where the Cloud TPU is located in. If not '
    'specified, we will attempt to automatically detect the GCE project from '
    'metadata.',
)

tf.flags.DEFINE_string(
    'gcp_project',
    None,
    '[Optional] Project name for the Cloud TPU-enabled project. If not '
    'specified, we will attempt to automatically detect the GCE project from '
    'metadata.',
)

tf.flags.DEFINE_string('master', None, '[Optional] TensorFlow master URL.')

flags.DEFINE_integer(
    'num_tpu_cores',
    8,
    'Only used if `use_tpu` is True. Total number of TPU cores to use.',
)

flags.DEFINE_float(
    'masked_lm_budget',
    0,
    'If >0, the ratio of masked ngrams to unmasked ngrams. Default 0,'
    'for offline masking',
)
flags.DEFINE_bool('use_gpu', False, 'Whether to use GPU.')

flags.DEFINE_integer(
    'num_gpu_cores',
    0,
    'Only used if `use_gpu` is True. Total number of GPU cores to use.',
)


def per_device_batch_size(batch_size, num_gpus):
    """For multi-gpu, batch-size must be a multiple of the number of GPUs.
  Note that this should eventually be handled by DistributionStrategies
  directly. Multi-GPU support is currently experimental, however,
  so doing the work here until that feature is in place.
  Args:
    batch_size: Global batch size to be divided among devices. This should be
      equal to num_gpus times the single-GPU batch_size for multi-gpu training.
    num_gpus: How many GPUs are used with DistributionStrategies.
  Returns:
    Batch size per device.
  Raises:
    ValueError: if batch_size is not divisible by number of devices
  """
    if num_gpus <= 1:
        return batch_size

    remainder = batch_size % num_gpus
    if remainder:
        err = (
            'When running with multiple GPUs, batch size '
            'must be a multiple of the number of available GPUs. Found {} '
            'GPUs with a batch size of {}; try --batch_size={} instead.'
        ).format(num_gpus, batch_size, batch_size - remainder)
        raise ValueError(err)
    return int(batch_size / num_gpus)


def model_fn_builder(
    albert_config,
    init_checkpoint,
    learning_rate,
    num_train_steps,
    num_warmup_steps,
    use_tpu,
    use_one_hot_embeddings,
    optimizer,
    poly_power,
    start_warmup_step,
):
    """Returns `model_fn` closure for TPUEstimator."""

    def model_fn(
        features, labels, mode, params
    ):  # pylint: disable=unused-argument
        """The `model_fn` for TPUEstimator."""

        tf.logging.info('*** Features ***')
        for name in sorted(features.keys()):
            tf.logging.info(
                '  name = %s, shape = %s' % (name, features[name].shape)
            )

        input_ids = features['input_ids']
        input_mask = features['input_mask']
        segment_ids = features['segment_ids']
        masked_lm_positions = features['masked_lm_positions']
        masked_lm_ids = features['masked_lm_ids']
        masked_lm_weights = features['masked_lm_weights']
        # Note: We keep this feature name `next_sentence_labels` to be compatible
        # with the original data created by lanzhzh@. However, in the ALBERT case
        # it does represent sentence_order_labels.
        sentence_order_labels = features['next_sentence_labels']

        is_training = mode == tf.estimator.ModeKeys.TRAIN

        model = modeling.AlbertModel(
            config = albert_config,
            is_training = is_training,
            input_ids = input_ids,
            input_mask = input_mask,
            token_type_ids = segment_ids,
            use_one_hot_embeddings = use_one_hot_embeddings,
        )

        (
            masked_lm_loss,
            masked_lm_example_loss,
            masked_lm_log_probs,
        ) = get_masked_lm_output(
            albert_config,
            model.get_sequence_output(),
            model.get_embedding_table(),
            masked_lm_positions,
            masked_lm_ids,
            masked_lm_weights,
        )

        (
            sentence_order_loss,
            sentence_order_example_loss,
            sentence_order_log_probs,
        ) = get_sentence_order_output(
            albert_config, model.get_pooled_output(), sentence_order_labels
        )

        total_loss = masked_lm_loss + sentence_order_loss

        tvars = tf.trainable_variables()

        initialized_variable_names = {}
        scaffold_fn = None
        if init_checkpoint:
            tf.logging.info(
                'number of hidden group %d to initialize',
                albert_config.num_hidden_groups,
            )
            num_of_initialize_group = 1
            if FLAGS.init_from_group0:
                num_of_initialize_group = albert_config.num_hidden_groups
                if albert_config.net_structure_type > 0:
                    num_of_initialize_group = albert_config.num_hidden_layers
            (
                assignment_map,
                initialized_variable_names,
            ) = modeling.get_assignment_map_from_checkpoint(
                tvars, init_checkpoint, num_of_initialize_group
            )
            if use_tpu:

                def tpu_scaffold():
                    for gid in range(num_of_initialize_group):
                        tf.logging.info('initialize the %dth layer', gid)
                        tf.logging.info(assignment_map[gid])
                        tf.train.init_from_checkpoint(
                            init_checkpoint, assignment_map[gid]
                        )
                    return tf.train.Scaffold()

                scaffold_fn = tpu_scaffold
            else:
                for gid in range(num_of_initialize_group):
                    tf.logging.info('initialize the %dth layer', gid)
                    tf.logging.info(assignment_map[gid])
                    tf.train.init_from_checkpoint(
                        init_checkpoint, assignment_map[gid]
                    )

        tf.logging.info('**** Trainable Variables ****')
        for var in tvars:
            init_string = ''
            if var.name in initialized_variable_names:
                init_string = ', *INIT_FROM_CKPT*'
            tf.logging.info(
                '  name = %s, shape = %s%s', var.name, var.shape, init_string
            )

        output_spec = None
        if mode == tf.estimator.ModeKeys.TRAIN:
            train_op = optimization.create_optimizer(
                total_loss, learning_rate, num_train_steps, num_warmup_steps
            )
            output_spec = tf.estimator.EstimatorSpec(
                mode = mode,
                loss = total_loss,
                train_op = train_op,
                scaffold = scaffold_fn,
            )

        elif mode == tf.estimator.ModeKeys.EVAL:

            def metric_fn(*args):
                """Computes the loss and accuracy of the model."""
                (
                    masked_lm_example_loss,
                    masked_lm_log_probs,
                    masked_lm_ids,
                    masked_lm_weights,
                    sentence_order_example_loss,
                    sentence_order_log_probs,
                    sentence_order_labels,
                ) = args[:7]

                masked_lm_log_probs = tf.reshape(
                    masked_lm_log_probs, [-1, masked_lm_log_probs.shape[-1]]
                )
                masked_lm_predictions = tf.argmax(
                    masked_lm_log_probs, axis = -1, output_type = tf.int32
                )
                masked_lm_example_loss = tf.reshape(
                    masked_lm_example_loss, [-1]
                )
                masked_lm_ids = tf.reshape(masked_lm_ids, [-1])
                masked_lm_weights = tf.reshape(masked_lm_weights, [-1])
                masked_lm_accuracy = tf.metrics.accuracy(
                    labels = masked_lm_ids,
                    predictions = masked_lm_predictions,
                    weights = masked_lm_weights,
                )
                masked_lm_mean_loss = tf.metrics.mean(
                    values = masked_lm_example_loss, weights = masked_lm_weights
                )

                metrics = {
                    'masked_lm_accuracy': masked_lm_accuracy,
                    'masked_lm_loss': masked_lm_mean_loss,
                }

                sentence_order_log_probs = tf.reshape(
                    sentence_order_log_probs,
                    [-1, sentence_order_log_probs.shape[-1]],
                )
                sentence_order_predictions = tf.argmax(
                    sentence_order_log_probs, axis = -1, output_type = tf.int32
                )
                sentence_order_labels = tf.reshape(sentence_order_labels, [-1])
                sentence_order_accuracy = tf.metrics.accuracy(
                    labels = sentence_order_labels,
                    predictions = sentence_order_predictions,
                )
                sentence_order_mean_loss = tf.metrics.mean(
                    values = sentence_order_example_loss
                )
                metrics.update(
                    {
                        'sentence_order_accuracy': sentence_order_accuracy,
                        'sentence_order_loss': sentence_order_mean_loss,
                    }
                )
                return metrics

            metric_values = [
                masked_lm_example_loss,
                masked_lm_log_probs,
                masked_lm_ids,
                masked_lm_weights,
                sentence_order_example_loss,
                sentence_order_log_probs,
                sentence_order_labels,
            ]

            eval_metrics = (metric_fn, metric_values)

            output_spec = contrib_tpu.TPUEstimatorSpec(
                mode = mode,
                loss = total_loss,
                eval_metrics = eval_metrics,
                scaffold_fn = scaffold_fn,
            )
        else:
            raise ValueError(
                'Only TRAIN and EVAL modes are supported: %s' % (mode)
            )

        return output_spec

    return model_fn


def get_masked_lm_output(
    albert_config,
    input_tensor,
    output_weights,
    positions,
    label_ids,
    label_weights,
):
    """Get loss and log probs for the masked LM."""
    input_tensor = gather_indexes(input_tensor, positions)

    with tf.variable_scope('cls/predictions'):
        # We apply one more non-linear transformation before the output layer.
        # This matrix is not used after pre-training.
        with tf.variable_scope('transform'):
            input_tensor = tf.layers.dense(
                input_tensor,
                units = albert_config.embedding_size,
                activation = modeling.get_activation(albert_config.hidden_act),
                kernel_initializer = modeling.create_initializer(
                    albert_config.initializer_range
                ),
            )
            input_tensor = modeling.layer_norm(input_tensor)

        # The output weights are the same as the input embeddings, but there is
        # an output-only bias for each token.
        output_bias = tf.get_variable(
            'output_bias',
            shape = [albert_config.vocab_size],
            initializer = tf.zeros_initializer(),
        )
        logits = tf.matmul(input_tensor, output_weights, transpose_b = True)
        logits = tf.nn.bias_add(logits, output_bias)
        log_probs = tf.nn.log_softmax(logits, axis = -1)

        label_ids = tf.reshape(label_ids, [-1])
        label_weights = tf.reshape(label_weights, [-1])

        one_hot_labels = tf.one_hot(
            label_ids, depth = albert_config.vocab_size, dtype = tf.float32
        )

        # The `positions` tensor might be zero-padded (if the sequence is too
        # short to have the maximum number of predictions). The `label_weights`
        # tensor has a value of 1.0 for every real prediction and 0.0 for the
        # padding predictions.
        per_example_loss = -tf.reduce_sum(
            log_probs * one_hot_labels, axis = [-1]
        )
        numerator = tf.reduce_sum(label_weights * per_example_loss)
        denominator = tf.reduce_sum(label_weights) + 1e-5
        loss = numerator / denominator

    return (loss, per_example_loss, log_probs)


def get_sentence_order_output(albert_config, input_tensor, labels):
    """Get loss and log probs for the next sentence prediction."""

    # Simple binary classification. Note that 0 is "next sentence" and 1 is
    # "random sentence". This weight matrix is not used after pre-training.
    with tf.variable_scope('cls/seq_relationship'):
        output_weights = tf.get_variable(
            'output_weights',
            shape = [2, albert_config.hidden_size],
            initializer = modeling.create_initializer(
                albert_config.initializer_range
            ),
        )
        output_bias = tf.get_variable(
            'output_bias', shape = [2], initializer = tf.zeros_initializer()
        )

        logits = tf.matmul(input_tensor, output_weights, transpose_b = True)
        logits = tf.nn.bias_add(logits, output_bias)
        log_probs = tf.nn.log_softmax(logits, axis = -1)
        labels = tf.reshape(labels, [-1])
        one_hot_labels = tf.one_hot(labels, depth = 2, dtype = tf.float32)
        per_example_loss = -tf.reduce_sum(one_hot_labels * log_probs, axis = -1)
        loss = tf.reduce_mean(per_example_loss)
        return (loss, per_example_loss, log_probs)


def gather_indexes(sequence_tensor, positions):
    """Gathers the vectors at the specific positions over a minibatch."""
    sequence_shape = modeling.get_shape_list(sequence_tensor, expected_rank = 3)
    batch_size = sequence_shape[0]
    seq_length = sequence_shape[1]
    width = sequence_shape[2]

    flat_offsets = tf.reshape(
        tf.range(0, batch_size, dtype = tf.int32) * seq_length, [-1, 1]
    )
    flat_positions = tf.reshape(positions + flat_offsets, [-1])
    flat_sequence_tensor = tf.reshape(
        sequence_tensor, [batch_size * seq_length, width]
    )
    output_tensor = tf.gather(flat_sequence_tensor, flat_positions)
    return output_tensor


def input_fn_builder(
    input_files,
    max_seq_length,
    max_predictions_per_seq,
    is_training,
    num_cpu_threads = 4,
):
    """Creates an `input_fn` closure to be passed to TPUEstimator."""

    def input_fn(params):
        """The actual input function."""
        batch_size = params['batch_size']

        name_to_features = {
            'input_ids': tf.FixedLenFeature([max_seq_length], tf.int64),
            'input_mask': tf.FixedLenFeature([max_seq_length], tf.int64),
            'segment_ids': tf.FixedLenFeature([max_seq_length], tf.int64),
            # Note: We keep this feature name `next_sentence_labels` to be
            # compatible with the original data created by lanzhzh@. However, in
            # the ALBERT case it does represent sentence_order_labels.
            'next_sentence_labels': tf.FixedLenFeature([1], tf.int64),
        }

        if FLAGS.masked_lm_budget:
            name_to_features.update(
                {
                    'token_boundary': tf.FixedLenFeature(
                        [max_seq_length], tf.int64
                    )
                }
            )
        else:
            name_to_features.update(
                {
                    'masked_lm_positions': tf.FixedLenFeature(
                        [max_predictions_per_seq], tf.int64
                    ),
                    'masked_lm_ids': tf.FixedLenFeature(
                        [max_predictions_per_seq], tf.int64
                    ),
                    'masked_lm_weights': tf.FixedLenFeature(
                        [max_predictions_per_seq], tf.float32
                    ),
                }
            )

        # For training, we want a lot of parallel reading and shuffling.
        # For eval, we want no shuffling and parallel reading doesn't matter.
        if is_training:
            d = tf.data.Dataset.from_tensor_slices(tf.constant(input_files))
            d = d.repeat()
            d = d.shuffle(buffer_size = len(input_files))

            # `cycle_length` is the number of parallel files that get read.
            cycle_length = min(num_cpu_threads, len(input_files))

            # `sloppy` mode means that the interleaving is not exact. This adds
            # even more randomness to the training pipeline.
            d = d.apply(
                contrib_data.parallel_interleave(
                    tf.data.TFRecordDataset,
                    sloppy = is_training,
                    cycle_length = cycle_length,
                )
            )
            d = d.shuffle(buffer_size = 100)
        else:
            d = tf.data.TFRecordDataset(input_files)
            # Since we evaluate for a fixed number of steps we don't want to encounter
            # out-of-range exceptions.
            d = d.repeat()

        # We must `drop_remainder` on training because the TPU requires fixed
        # size dimensions. For eval, we assume we are evaluating on the CPU or GPU
        # and we *don't* want to drop the remainder, otherwise we wont cover
        # every sample.
        d = d.apply(
            tf.data.experimental.map_and_batch_with_legacy_function(
                lambda record: _decode_record(record, name_to_features),
                batch_size = batch_size,
                num_parallel_batches = num_cpu_threads,
                drop_remainder = True,
            )
        )
        tf.logging.info(d)
        return d

    return input_fn


def input_fn_builder_gpu(
    input_files,
    max_seq_length,
    max_predictions_per_seq,
    is_training,
    batch_size,
    num_cpu_threads = 4,
):
    """Creates an `input_fn` closure to be passed to TPUEstimator."""

    def input_fn(params):

        name_to_features = {
            'input_ids': tf.FixedLenFeature([max_seq_length], tf.int64),
            'input_mask': tf.FixedLenFeature([max_seq_length], tf.int64),
            'segment_ids': tf.FixedLenFeature([max_seq_length], tf.int64),
            'masked_lm_positions': tf.FixedLenFeature(
                [max_predictions_per_seq], tf.int64
            ),
            'masked_lm_ids': tf.FixedLenFeature(
                [max_predictions_per_seq], tf.int64
            ),
            'masked_lm_weights': tf.FixedLenFeature(
                [max_predictions_per_seq], tf.float32
            ),
            'next_sentence_labels': tf.FixedLenFeature([1], tf.int64),
        }

        # For training, we want a lot of parallel reading and shuffling.
        # For eval, we want no shuffling and parallel reading doesn't matter.
        if is_training:
            d = tf.data.Dataset.from_tensor_slices(tf.constant(input_files))
            d = d.repeat()
            d = d.shuffle(buffer_size = len(input_files))

            # `cycle_length` is the number of parallel files that get read.
            cycle_length = min(num_cpu_threads, len(input_files))

            # `sloppy` mode means that the interleaving is not exact. This adds
            # even more randomness to the training pipeline.
            d = d.apply(
                tf.contrib.data.parallel_interleave(
                    tf.data.TFRecordDataset,
                    sloppy = is_training,
                    cycle_length = cycle_length,
                )
            )
            d = d.shuffle(buffer_size = 100)
        else:
            d = tf.data.TFRecordDataset(input_files)
            # Since we evaluate for a fixed number of steps we don't want to encounter
            # out-of-range exceptions.
            d = d.repeat()

        # We must `drop_remainder` on training because the TPU requires fixed
        # size dimensions. For eval, we assume we are evaluating on the CPU or GPU
        # and we *don't* want to drop the remainder, otherwise we wont cover
        # every sample.
        d = d.apply(
            tf.contrib.data.map_and_batch(
                lambda record: _decode_record(record, name_to_features),
                batch_size = batch_size,
                num_parallel_batches = num_cpu_threads,
                drop_remainder = True,
            )
        )
        return d

    return input_fn


def _decode_record(record, name_to_features):
    """Decodes a record to a TensorFlow example."""
    example = tf.parse_single_example(record, name_to_features)

    # tf.Example only supports tf.int64, but the TPU only supports tf.int32.
    # So cast all int64 to int32.
    for name in list(example.keys()):
        t = example[name]
        if t.dtype == tf.int64:
            t = tf.to_int32(t)
        example[name] = t

    return example


def main(_):
    tf.logging.set_verbosity(tf.logging.INFO)

    if not FLAGS.do_train and not FLAGS.do_eval:
        raise ValueError(
            'At least one of `do_train` or `do_eval` must be True.'
        )

    albert_config = modeling.AlbertConfig.from_json_file(
        FLAGS.albert_config_file
    )

    tf.gfile.MakeDirs(FLAGS.output_dir)

    input_files = []
    for input_pattern in FLAGS.input_file.split(','):
        input_files.extend(tf.gfile.Glob(input_pattern))

    tf.logging.info('*** Input Files ***')
    for input_file in input_files:
        tf.logging.info('  %s' % input_file)

    tf.logging.info('Use normal RunConfig')
    tf.logging.info(FLAGS.num_gpu_cores)
    dist_strategy = tf.contrib.distribute.MirroredStrategy(
        num_gpus = FLAGS.num_gpu_cores,
        auto_shard_dataset = True,
        cross_device_ops = AllReduceCrossDeviceOps(
            'nccl', num_packs = FLAGS.num_gpu_cores
        ),
        # cross_device_ops=AllReduceCrossDeviceOps('hierarchical_copy'),
    )
    log_every_n_steps = 10
    run_config = RunConfig(
        train_distribute = dist_strategy,
        eval_distribute = dist_strategy,
        log_step_count_steps = log_every_n_steps,
        model_dir = FLAGS.output_dir,
        save_checkpoints_steps = FLAGS.save_checkpoints_steps,
        save_summary_steps = None,
    )

    model_fn = model_fn_builder(
        albert_config = albert_config,
        init_checkpoint = FLAGS.init_checkpoint,
        learning_rate = FLAGS.learning_rate,
        num_train_steps = FLAGS.num_train_steps,
        num_warmup_steps = FLAGS.num_warmup_steps,
        use_tpu = FLAGS.use_tpu,
        use_one_hot_embeddings = FLAGS.use_tpu,
        optimizer = FLAGS.optimizer,
        poly_power = FLAGS.poly_power,
        start_warmup_step = FLAGS.start_warmup_step,
    )

    tf.logging.info('Use normal Estimator')
    estimator = Estimator(model_fn = model_fn, params = {}, config = run_config)

    if FLAGS.do_train:
        tf.logging.info('***** Running training *****')
        tf.logging.info('  Batch size = %d', FLAGS.train_batch_size)
        train_input_fn = input_fn_builder_gpu(
            input_files = input_files,
            max_seq_length = FLAGS.max_seq_length,
            max_predictions_per_seq = FLAGS.max_predictions_per_seq,
            is_training = True,
            batch_size = per_device_batch_size(
                FLAGS.train_batch_size, FLAGS.num_gpu_cores
            ),
        )
        estimator.train(
            input_fn = train_input_fn, max_steps = FLAGS.num_train_steps
        )

    if FLAGS.do_eval:
        tf.logging.info('***** Running evaluation *****')
        tf.logging.info('  Batch size = %d', FLAGS.eval_batch_size)
        global_step = -1
        output_eval_file = os.path.join(FLAGS.output_dir, 'eval_results.txt')
        writer = tf.gfile.GFile(output_eval_file, 'w')
        tf.gfile.MakeDirs(FLAGS.export_dir)
        eval_input_fn = input_fn_builder(
            input_files = input_files,
            max_seq_length = FLAGS.max_seq_length,
            max_predictions_per_seq = FLAGS.max_predictions_per_seq,
            is_training = False,
        )
        while global_step < FLAGS.num_train_steps:
            if estimator.latest_checkpoint() is None:
                tf.logging.info('No checkpoint found yet. Sleeping.')
                time.sleep(1)
            else:
                result = estimator.evaluate(
                    input_fn = eval_input_fn, steps = FLAGS.max_eval_steps
                )
                global_step = result['global_step']
                tf.logging.info('***** Eval results *****')
                for key in sorted(result.keys()):
                    tf.logging.info('  %s = %s', key, str(result[key]))
                    writer.write('%s = %s\n' % (key, str(result[key])))


if __name__ == '__main__':
    flags.mark_flag_as_required('input_file')
    flags.mark_flag_as_required('albert_config_file')
    flags.mark_flag_as_required('output_dir')
    tf.app.run()



import malaya
import json
from fastapi import FastAPI

app = FastAPI()

model = malaya.sentiment.transformer(
    model = 'albert', size = 'base', validate = False
)


@app.get('/')
def index(string: str = None):
    strings = [string] * 50
    r = model.predict_batch(strings, get_proba = True)
    return json.dumps('done')



english = {
    'afaik': 'as far as i know',
    'afk': 'away from keyboard',
    'asap': 'as soon as possible',
    'atk': 'at the keyboard',
    'atm': 'at the moment',
    'a3': 'anytime, anywhere, anyplace',
    'bak': 'back at keyboard',
    'bbl': 'be back later',
    'bbs': 'be back soon',
    'bfn': 'bye for now',
    'b4n': 'bye for now',
    'brb': 'be right back',
    'brt': 'be right there',
    'btw': 'by the way',
    'b4': 'before',
    'cu': 'see you',
    'cul8r': 'see you later',
    'cya': 'see you',
    'faq': 'frequently asked questions',
    'fc': 'fingers crossed',
    'fwiw': 'for what its worth',
    'fyi': 'for your information',
    'gal': 'get a life',
    'gg': 'good game',
    'gn': 'good night',
    'gmta': 'great minds think alike',
    'gr8': 'great!',
    'g9': 'genius',
    'ic': 'i see',
    'icq': 'i seek you',
    'ilu': 'ilu: i love you',
    'imho': 'in my humble opinion',
    'imo': 'in my opinion',
    'iow': 'in other words',
    'irl': 'in real life',
    'kiss': 'keep it simple, stupid',
    'ldr': 'long distance relationship',
    'lmao': 'laugh my ass off',
    'lol': 'laughing out loud',
    'ltns': 'long time no see',
    'l8r': 'later',
    'mte': 'my thoughts exactly',
    'm8': 'mate',
    'nrn': 'no reply necessary',
    'oic': 'oh i see',
    'pita': 'pain in the ass',
    'prt': 'party',
    'prw': 'parents are watching',
    'qpsa': 'que pasa',
    'rofl': 'rolling on the floor laughing',
    'roflol': 'rolling on the floor laughing out loud',
    'rotflmao': 'rolling on the floor laughing my ass off',
    'sk8': 'skate',
    'stats': 'your sex and age',
    'asl': 'age sex location',
    'thx': 'thank you',
    'ty': 'thank you',
    'ttfn': 'ta-ta for now!',
    'ttyl': 'talk to you later',
    'u': 'you',
    'u2': 'you too',
    'u4e': 'yours for ever',
    'wb': 'welcome back',
    'wtf': 'what the fuck',
    'wtg': 'way to go!',
    'wuf': 'where are you from?',
    'w8': 'wait',
    '7k': 'sick laugher',
    'r': 'are',
    'w': 'wait',
    'hrs': 'hours',
    'd': 'the',
    'fwd': 'forward',
    'cant': 'cannot',
    'hve': 'have',
    'cause': 'because',
    'couldve': 'could have',
    've': 'have',
    'nt': 'not',
    'gonna': 'going to',
    'id': 'i would',
    'maam': 'madam',
    'nothin': 'nothing',
    'fu': 'fuck you',
    'aamof': 'as a matter of fact',
    'ack': 'acknowledgment',
    'ot': 'off topic',
    'op': 'original poster original post',
    'otoh': 'on the other hand',
    'pov': 'point of view',
    'rotfl': 'rolling on the floor laughing',
    'rtfm': 'read the fine manual',
    'fck': 'fuck',
    'tyvm': 'thank you very much',
    'tyt': 'take your time',
    'yeo fyeo': 'for your eyes only',
    'afair': 'as far as i remember recall',
    'aka': 'also known as',
    'bkbtk': 'back to keyboard',
    'btt': 'back to topic',
    'bc': 'because',
    'cys': 'check your settings',
    'diy': 'do it yourself',
    'eobd': 'end of business day',
    'eod': 'end of day',
    'eom': 'end of message',
    'eot': 'end of thread text transmission',
    'fack': 'full acknowledge',
    'fka': 'formerly known as',
    'fyi jfyi': 'justfor your information',
    'ftw': 'fuck the world for the win',
    'hf': 'have fun',
    'hth': 'hope this helps',
    'idk': 'i dont know',
    'iirc': 'if i recall remember correctly',
    'imnsho': 'in my not so humble honest opinion',
    'itt': 'in this thread',
    'dgmw': 'dont get me wrong',
    'mmw': 'mark my words',
    'na': 'not available applicable',
    'nan': 'not a number',
    'nntr': 'no need to reply',
    'noyb': 'none of your business',
    'omg': 'oh my god',
    'pebkac': 'problem exists between keyboard and chair',
    'rsvp repondez sil vous plait french': 'please reply',
    'scnr': 'sorry could not resist',
    'sflr': 'sorry for late reply',
    'spoc': 'single point of contact',
    'tba': 'to be announced',
    'tbc': 'to be continued to be confirmed',
    'tia': 'thanks in advance',
    'tgif': 'thanks god its friday',
    'thx tnx': 'thanks',
    'tq': 'thank you',
    'wt': 'whoomp there it is',
    'wfm': 'works for me',
    'wrt': 'with regard to',
    'wth': 'what the hell what the heck',
    'ymmd': 'you made my day',
    'ymmv': 'your mileage may vary',
    'yam': 'yet another meeting',
    'icymi': 'in case you missed it',
    'moro': 'tomorrow',
    'nte': 'tonight',
    'aeap': 'as early as possible',
    'alap': 'as late as possible',
    'b': 'blah blah blah',
    'byki': 'before you know it',
    'bff': 'best friends forever',
    'btam': 'be that as it may',
    'ctn': 'cannot talk now',
    'cus': 'see you soon',
    'cwot': 'complete waste of time',
    'cyt': 'see you tomorrow',
    'em': 'excuse me',
    'ff': 'face to face',
    'foaf': 'friend of a friend',
    'gr': 'great',
    'hak': 'hugs and kisses',
    'idc': 'i dont care',
    'ilu ily': 'i love you',
    'imu': 'i miss you',
    'jk': 'just kidding',
    'jc': 'just checking',
    'jtlyk': 'just to let you know',
    'kfy': 'kiss for you',
    'kmn': 'kill me now',
    'kpc': 'keeping parents clueless',
    'lr': 'later',
    'mof': 'male or female',
    'mtfbwy': 'may the force be with you',
    'myob': 'mind your own business',
    'naz': 'name address zip',
    'nc': 'no comment',
    'nimby': 'not in my backyard',
    'nm': 'never mind nothing much',
    'np': 'no problem',
    'nsfw': 'not safe for work',
    'ntim': 'not that it matters',
    'nvm': 'never mind',
    'oatus': 'on a totally unrelated subject',
    'omw': 'on my way',
    'otl': 'out to lunch',
    'otp': 'on the phone',
    'p': 'parent alert',
    'pal': 'parents are listening',
    'paw': 'parents are watching',
    'pir': 'parent in room',
    'pos': 'parent over shoulder',
    'props': 'proper respect proper recognition',
    'qt': 'cutie',
    'rn': 'right now',
    'ru': 'are you',
    'sep': 'someone elses problem',
    'sitd': 'still in the dark',
    'slap': 'sounds like a plan',
    'smim': 'send me an instant message',
    'so': 'significant other',
    'tmi': 'too much information',
    'ur': 'your you are',
    'wycm': 'will you call me',
    'wywh': 'wish you were here',
    'xoxoxox': 'hugs kisses',
}



import sys
import threading
import webbrowser
import socket
import itertools
import random
import os
import mimetypes
from http import server


def generate_handler(html, files = None):
    if files is None:
        files = {}

    class MyHandler(server.BaseHTTPRequestHandler):
        def do_GET(self):
            self.send_response(200)

            if self.path == '/':
                self.send_header('Content-type', 'text/html')
                self.end_headers()
                self.wfile.write(html.encode('utf-8'))
            else:
                this_dir = os.path.dirname(__file__)
                filepath = os.path.join(this_dir, 'web', self.path[1:])

                mimetype, _ = mimetypes.guess_type(filepath)
                self.send_header('Content-type', mimetype)
                self.end_headers()

                with open(filepath, 'rb') as fh:
                    content = fh.read()
                self.wfile.write(content)

    return MyHandler


def find_open_port(ip, port, n = 50):
    """
    Find an open port near the specified port
    """
    ports = itertools.chain(
        (port + i for i in range(n)), (port + random.randint(-2 * n, 2 * n))
    )

    for port in ports:
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        result = s.connect_ex((ip, port))
        s.close()
        if result != 0:
            return port
    raise ValueError('no open ports found')


def serve(
    html,
    ip = '127.0.0.1',
    port = 8888,
    n_retries = 50,
    files = None,
    ipython_warning = False,
    open_browser = True,
    http_server = None,
):
    """
    Start a server serving the given HTML, and (optionally) open a browser.

    Parameters
    ----------
    html : string
        HTML to serve
    ip : string (default = '127.0.0.1')
        ip address at which the HTML will be served.
    port : int (default = 8888)
        the port at which to serve the HTML
    n_retries : int (default = 50)
        the number of nearby ports to search if the specified port is in use.
    files : dictionary (optional)
        dictionary of extra content to serve
    ipython_warning : bool (optional)
        if True (default), then print a warning if this is used within IPython
    open_browser : bool (optional)
        if True (default), then open a web browser to the given HTML
    http_server : class (optional)
        optionally specify an HTTPServer class to use for showing the
        figure. The default is Python's basic HTTPServer.
    """

    port = find_open_port(ip, port, n_retries)
    Handler = generate_handler(html, files)

    if http_server is None:
        srvr = server.HTTPServer((ip, port), Handler)
    else:
        srvr = http_server((ip, port), Handler)

    if ipython_warning:
        print(IPYTHON_WARNING)

    print('Serving to http://{0}:{1}/    [Ctrl-C to exit]'.format(ip, port))
    sys.stdout.flush()

    if open_browser:
        b = lambda: webbrowser.open('http://{0}:{1}'.format(ip, port))
        threading.Thread(target = b).start()

    try:
        srvr.serve_forever()
    except (KeyboardInterrupt, SystemExit):
        print('\nstopping Server...')

    srvr.server_close()



import numpy as np
import json
import re
import dateparser
from unidecode import unidecode
from .num2word import to_cardinal, to_ordinal
from .word2num import word2num
from .texts._text_functions import ENGLISH_WORDS, MALAY_WORDS, multireplace
from .texts._regex import (
    _date,
    _past_date_string,
    _now_date_string,
    _future_date_string,
    _yesterday_tomorrow_date_string,
    _depan_date_string,
    _money,
    _expressions,
    _left_datetime,
    _right_datetime,
    _today_time,
    _left_datetodaytime,
    _right_datetodaytime,
    _left_yesterdaydatetime,
    _right_yesterdaydatetime,
    _left_yesterdaydatetodaytime,
    _right_yesterdaydatetodaytime,
)
from .texts._tatabahasa import (
    rules_normalizer,
    date_replace,
    consonants,
    sounds,
    hujung_malaysian,
)
from .texts._normalization import (
    _remove_postfix,
    _normalize_title,
    _is_number_regex,
    _string_to_num,
    _normalize_money,
    cardinal,
    digit,
    rom_to_int,
    ordinal,
    fraction,
    money,
    ignore_words,
)
from .cluster import cluster_words
from .preprocessing import _tokenizer


class _SPELL_NORMALIZE:
    def __init__(self, speller):
        self._speller = speller

    def normalize(self, string, check_english = True):
        """
        Normalize a string

        Parameters
        ----------
        string : str
        check_english: bool, (default=True)
            check a word in english dictionary.

        Returns
        -------
        string: normalized string
        """
        if not isinstance(string, str):
            raise ValueError('input must be a string')
        if not isinstance(check_english, bool):
            raise ValueError('check_english must be a boolean')

        result, normalized = [], []
        tokenized = _tokenizer(string)
        index = 0
        while index < len(tokenized):
            word = tokenized[index]
            if word in '~@#$%^&*()_+{}|[:"\'];<>,.?/-':
                result.append(word)
                index += 1
                continue
            normalized.append(rules_normalizer.get(word.lower(), word.lower()))
            if word.lower() in ignore_words:
                result.append(word)
                index += 1
                continue
            if word[0].isupper():
                if word.upper() not in ['KE', 'PADA', 'RM', 'SEN', 'HINGGA']:
                    result.append(_normalize_title(word))
                    index += 1
                    continue
            if check_english:
                if word.lower() in ENGLISH_WORDS:
                    result.append(word)
                    index += 1
                    continue
            if word.lower() in MALAY_WORDS and word.lower() not in [
                'pada',
                'ke',
            ]:
                result.append(word)
                index += 1
                continue
            if len(word) > 2:
                if word[-2] in consonants and word[-1] == 'e':
                    word = word[:-1] + 'a'
            if word[0] == 'x' and len(word) > 1:
                result_string = 'tak '
                word = word[1:]
            else:
                result_string = ''

            if word.lower() == 'ke' and index < (len(tokenized) - 2):
                if tokenized[index + 1] == '-' and _is_number_regex(
                    tokenized[index + 2]
                ):
                    result.append(
                        ordinal(
                            word + tokenized[index + 1] + tokenized[index + 2]
                        )
                    )
                    index += 3
                    continue
                elif tokenized[index + 1] == '-' and re.match(
                    '.*(V|X|I|L|D)', tokenized[index + 2]
                ):
                    result.append(
                        ordinal(
                            word
                            + tokenized[index + 1]
                            + str(rom_to_int(tokenized[index + 2]))
                        )
                    )
                    index += 3
                    continue
                else:
                    result.append('ke')
                    index += 1
                    continue

            if _is_number_regex(word) and index < (len(tokenized) - 2):
                if tokenized[index + 1] == '-' and _is_number_regex(
                    tokenized[index + 2]
                ):
                    result.append(
                        to_cardinal(_string_to_num(word))
                        + ' hingga '
                        + to_cardinal(_string_to_num(tokenized[index + 2]))
                    )
                    index += 3
                    continue
            if word.lower() == 'pada' and index < (len(tokenized) - 3):
                if (
                    _is_number_regex(tokenized[index + 1])
                    and tokenized[index + 2] in '/-'
                    and _is_number_regex(tokenized[index + 3])
                ):
                    result.append(
                        'pada %s hari bulan %s'
                        % (
                            to_cardinal(_string_to_num(tokenized[index + 1])),
                            to_cardinal(_string_to_num(tokenized[index + 3])),
                        )
                    )
                    index += 4
                    continue
                else:
                    result.append('pada')
                    index += 1
                    continue

            if _is_number_regex(word) and index < (len(tokenized) - 2):
                if tokenized[index + 1] == '/' and _is_number_regex(
                    tokenized[index + 2]
                ):
                    result.append(
                        fraction(
                            word + tokenized[index + 1] + tokenized[index + 2]
                        )
                    )
                    index += 3
                    continue

            if re.findall(_money, word.lower()):
                money_, _ = money(word)
                result.append(money_)
                index += 1
                continue

            if re.findall(_date, word.lower()):
                word = word.lower()
                word = multireplace(word, date_replace)
                word = re.sub(r'[ ]+', ' ', word).strip()
                parsed = dateparser.parse(word)
                if parsed:
                    result.append(parsed.strftime('%d/%m/%Y'))
                else:
                    result.append(word)
                index += 1
                continue

            if re.findall(_expressions['time'], word.lower()):
                word = word.lower()
                word = multireplace(word, date_replace)
                word = re.sub(r'[ ]+', ' ', word).strip()
                parsed = dateparser.parse(word)
                if parsed:
                    result.append(parsed.strftime('%H:%M:%S'))
                else:
                    result.append(word)
                index += 1
                continue

            cardinal_ = cardinal(word)
            if cardinal_ != word:
                result.append(cardinal_)
                index += 1
                continue

            normalized_ke = ordinal(word)
            if normalized_ke != word:
                result.append(normalized_ke)
                index += 1
                continue
            word, end_result_string = _remove_postfix(word)
            if word in sounds:
                result.append(result_string + sounds[word] + end_result_string)
                index += 1
                continue
            if word in rules_normalizer:
                result.append(
                    result_string + rules_normalizer[word] + end_result_string
                )
                index += 1
                continue
            selected = self._speller.correct(
                word, string = ' '.join(tokenized), index = index
            )
            result.append(result_string + selected + end_result_string)
            index += 1

        result = ' '.join(result)
        normalized = ' '.join(normalized)
        money_ = re.findall(_money, normalized)
        money_ = [(s, money(s)[1]) for s in money_]
        dates_ = re.findall(_date, normalized)

        past_date_string_ = re.findall(_past_date_string, normalized)
        now_date_string_ = re.findall(_now_date_string, normalized)
        future_date_string_ = re.findall(_future_date_string, normalized)
        yesterday_date_string_ = re.findall(
            _yesterday_tomorrow_date_string, normalized
        )
        depan_date_string_ = re.findall(_depan_date_string, normalized)
        today_time_ = re.findall(_today_time, normalized)
        time_ = re.findall(_expressions['time'], normalized)
        left_datetime_ = [
            '%s %s' % (i[0], i[1])
            for i in re.findall(_left_datetime, normalized)
        ]
        right_datetime_ = [
            '%s %s' % (i[0], i[1])
            for i in re.findall(_right_datetime, normalized)
        ]
        today_left_datetime_ = [
            '%s %s' % (i[0], i[1])
            for i in re.findall(_left_datetodaytime, normalized)
        ]
        today_right_datetime_ = [
            '%s %s' % (i[0], i[1])
            for i in re.findall(_right_datetodaytime, normalized)
        ]
        left_yesterdaydatetime_ = [
            '%s %s' % (i[0], i[1])
            for i in re.findall(_left_yesterdaydatetime, normalized)
        ]
        right_yesterdaydatetime_ = [
            '%s %s' % (i[0], i[1])
            for i in re.findall(_right_yesterdaydatetime, normalized)
        ]
        left_yesterdaydatetodaytime_ = [
            '%s %s' % (i[0], i[1])
            for i in re.findall(_left_yesterdaydatetodaytime, normalized)
        ]
        right_yesterdaydatetodaytime_ = [
            '%s %s' % (i[0], i[1])
            for i in re.findall(_right_yesterdaydatetodaytime, normalized)
        ]

        dates_ = (
            dates_
            + past_date_string_
            + now_date_string_
            + future_date_string_
            + yesterday_date_string_
            + depan_date_string_
            + time_
            + today_time_
            + left_datetime_
            + right_datetime_
            + today_left_datetime_
            + today_right_datetime_
            + left_yesterdaydatetime_
            + right_yesterdaydatetime_
            + left_yesterdaydatetodaytime_
            + right_yesterdaydatetodaytime_
        )
        dates_ = [multireplace(s, date_replace) for s in dates_]
        dates_ = [re.sub(r'[ ]+', ' ', s).strip() for s in dates_]
        dates_ = cluster_words(dates_)
        dates_ = {s: dateparser.parse(s) for s in dates_}
        money_ = {s[0]: s[1] for s in money_}
        return {'normalize': result, 'date': dates_, 'money': money_}


def spell(speller):
    """
    Train a Spelling Normalizer

    Parameters
    ----------
    speller : Malaya spelling correction object

    Returns
    -------
    _SPELL_NORMALIZE: malaya.normalizer._SPELL_NORMALIZE class
    """
    if not hasattr(speller, 'correct') and not hasattr(
        speller, 'normalize_elongated'
    ):
        raise ValueError(
            'speller must has `correct` or `normalize_elongated` method'
        )
    return _SPELL_NORMALIZE(speller)



import sklearn.datasets
import numpy as np
import re
import collections
from sklearn import metrics

def clearstring(string):
    string = re.sub('[^A-Za-z0-9 ]+', '', string)
    string = string.split(' ')
    string = filter(None, string)
    string = [y.strip() for y in string]
    string = ' '.join(string)
    return string.lower()

def separate_dataset(trainset,ratio=0.5):
    datastring = []
    datatarget = []
    for i in range(int(len(trainset.data)*ratio)):
        data_ = trainset.data[i].split('\n')
        data_ = list(filter(None, data_))
        for n in range(len(data_)):
            data_[n] = clearstring(data_[n])
        datastring += data_
        for n in range(len(data_)):
            datatarget.append(trainset.target[i])
    return datastring, datatarget

def build_dataset(words, n_words):
    count = [['GO', 0], ['PAD', 1], ['EOS', 2], ['UNK', 3]]
    count.extend(collections.Counter(words).most_common(n_words - 1))
    dictionary = dict()
    for word, _ in count:
        dictionary[word] = len(dictionary)
    data = list()
    unk_count = 0
    for word in words:
        index = dictionary.get(word, 0)
        if index == 0:
            unk_count += 1
        data.append(index)
    count[0][1] = unk_count
    reversed_dictionary = dict(zip(dictionary.values(), dictionary.keys()))
    return data, count, dictionary, reversed_dictionary

def str_idx(corpus, dic, maxlen, UNK=3):
    X = np.zeros((len(corpus),maxlen))
    for i in range(len(corpus)):
        for no, k in enumerate(corpus[i].split()[:maxlen][::-1]):
            try:
                X[i,-1 - no]=dic[k]
            except Exception as e:
                X[i,-1 - no]=UNK
    return X